# -*- coding: utf-8 -*-

import os
from io import BytesIO

import oss2
import unittest
import unittests
import random
from oss2 import utils

from functools import partial
from mock import patch
import struct
import copy
from .common import *
import json


def make_get_object(content):
    request_text = '''GET /sjbhlsgsbecvlpbf HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:PAedG7U86ZxQ2WTB+GdpSltoiTI='''

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Type: text/plain
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
Accept-Ranges: bytes
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
Last-Modified: Sat, 12 Dec 2015 00:35:53 GMT
x-oss-object-type: Normal

{1}'''.format(len(content), oss2.to_string(content))

    return request_text, response_text


def make_put_encrypted_object(key, content, content_crypto_material):
    cipher = content_crypto_material.cipher
    encrypted_key = utils.b64encode_as_string(content_crypto_material.encrypted_key)
    encrypted_iv = utils.b64encode_as_string(content_crypto_material.encrypted_iv)
    encrypted_content = cipher.encrypt(content)
    wrap_alg = content_crypto_material.wrap_alg
    cek_alg = content_crypto_material.cek_alg

    request_text = '''PUT /{0} HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: {1}
x-oss-meta-client-side-encryption-wrap-alg: {2}
x-oss-meta-client-side-encryption-cek-alg: {3}
x-oss-meta-client-side-encryption-key: {4}
x-oss-meta-client-side-encryption-start: {5}
x-oss-meta-unencrypted-content-length: {6}
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:W6whAowN4aImQ0dfbMHyFfD0t1g=
Accept: */*

'''.format(key, len(content), wrap_alg, cek_alg, encrypted_key, encrypted_iv, len(encrypted_content))

    io = BytesIO()
    io.write(oss2.to_bytes(request_text))
    io.write(encrypted_content)

    request_text = io.getvalue()

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
x-oss-hash-crc64ecma: {1}
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"'''.format(len(content), unittests.common.calc_crc(encrypted_content))

    return request_text, response_text


def make_get_encrypted_object(key, content, content_crypto_material, invalid_cek_alg='', ranges=None):
    request_text = '''GET /{0} HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:PAedG7U86ZxQ2WTB+GdpSltoiTI='''.format(key)

    encrypted_key = utils.b64encode_as_string(content_crypto_material.encrypted_key)
    encrypted_iv = utils.b64encode_as_string(content_crypto_material.encrypted_iv)
    wrap_alg = content_crypto_material.wrap_alg
    cek_alg = content_crypto_material.cek_alg
    if invalid_cek_alg:
        cek_alg = invalid_cek_alg
    cipher = content_crypto_material.cipher

    encrypted_content = cipher.encrypt(content)

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Type: text/plain
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
Accept-Ranges: bytes
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
x-oss-meta-client-side-encryption-wrap-alg: {1}
x-oss-meta-client-side-encryption-cek-alg: {2}
x-oss-meta-client-side-encryption-key: {3}
x-oss-meta-client-side-encryption-start: {4}
x-oss-meta-unencrypted-content-length: {5}
Last-Modified: Sat, 12 Dec 2015 00:35:53 GMT
x-oss-object-type: Normal{6}

'''.format(len(encrypted_content), wrap_alg, cek_alg, encrypted_key, encrypted_iv, len(content),
           '\nContent-Range: {0}'.format(ranges) if ranges else '')

    io = BytesIO()
    io.write(oss2.to_bytes(response_text))
    io.write(encrypted_content)

    response_text = io.getvalue()

    return request_text, response_text


def make_get_encrypted_object_compact(key, encrypted_content, encrypted_meta):
    request_text = '''GET /{0} HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:PAedG7U86ZxQ2WTB+GdpSltoiTI='''.format(key)

    encrypted_key = encrypted_meta['x-oss-meta-client-side-encryption-key']
    encrypted_iv = encrypted_meta['x-oss-meta-client-side-encryption-start']
    wrap_alg = encrypted_meta['x-oss-meta-client-side-encryption-wrap-alg']
    cek_alg = encrypted_meta['x-oss-meta-client-side-encryption-cek-alg']

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Type: text/plain
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
Accept-Ranges: bytes
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
x-oss-meta-client-side-encryption-wrap-alg: {1}
x-oss-meta-client-side-encryption-cek-alg: {2}
x-oss-meta-client-side-encryption-key: {3}
x-oss-meta-client-side-encryption-start: {4}
x-oss-meta-unencrypted-content-length: {5}
Last-Modified: Sat, 12 Dec 2015 00:35:53 GMT
x-oss-object-type: Normal

'''.format(len(encrypted_content), wrap_alg, cek_alg, encrypted_key, encrypted_iv, len(encrypted_content))

    io = BytesIO()
    io.write(oss2.to_bytes(response_text))
    io.write(encrypted_content)

    response_text = io.getvalue()

    return request_text, response_text


def make_get_encrypted_object_compact_deprecated(key, encrypted_content, encrypted_meta):
    request_text = '''GET /{0} HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:PAedG7U86ZxQ2WTB+GdpSltoiTI='''.format(key)

    encrypted_key = encrypted_meta['x-oss-meta-oss-crypto-key']
    encrypted_iv = encrypted_meta['x-oss-meta-oss-crypto-start']
    wrap_alg = encrypted_meta['x-oss-meta-oss-wrap-alg']
    cek_alg = encrypted_meta['x-oss-meta-oss-cek-alg']

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Type: text/plain
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
Accept-Ranges: bytes
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
x-oss-meta-oss-wrap-alg: {1}
x-oss-meta-oss-cek-alg: {2}
x-oss-meta-oss-crypto-key: {3}
x-oss-meta-oss-crypto-start: {4}
x-oss-meta-unencrypted-content-length: {5}
Last-Modified: Sat, 12 Dec 2015 00:35:53 GMT
x-oss-object-type: Normal

'''.format(len(encrypted_content), wrap_alg, cek_alg, encrypted_key, encrypted_iv, len(encrypted_content))

    io = BytesIO()
    io.write(oss2.to_bytes(response_text))
    io.write(encrypted_content)

    response_text = io.getvalue()

    return request_text, response_text


def make_put_object(content):
    request_text = '''PUT /sjbhlsgsbecvlpbf.txt HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Type: text/plain
Content-Length: {0}
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:W6whAowN4aImQ0dfbMHyFfD0t1g=
Accept: */*

{1}'''.format(len(content), oss2.to_string(content))

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
x-oss-hash-crc64ecma: {0}
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"'''.format(unittests.common.calc_crc(content))

    return request_text, response_text


def make_append_object(position, content):
    request_text = '''POST /sjbhlsgsbecvlpbf?position={0}&append= HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: {1}
date: Sat, 12 Dec 2015 00:36:29 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:1njpxsTivMNvTdfYolCUefRInVY=

{2}'''.format(position, len(content), oss2.to_string(content))

    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:36:29 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6C0D1790CF586F72240B
ETag: "24F7FA10676D816E0D6C6B5600000000"
x-oss-next-append-position: {0}
x-oss-hash-crc64ecma: {1}'''.format(position + len(content), unittests.common.calc_crc(content))

    return request_text, response_text

def make_get_object_tagging():
    request_text = '''GET /sjbhlsgsbecvlpbf?tagging HTTP/1.1
        Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
        Accept-Encoding: identity
        Connection: keep-alive
        date: Sat, 12 Dec 2015 00:35:53 GMT
        User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
        Accept: */*
                    authorization: OSS ZCDmm7TPZKHtx77j:PAedG7U86ZxQ2WTB+GdpSltoiTI='''

    response_text = '''HTTP/1.1 200 OK
                    Server: AliyunOSS
                    Date: Sat, 12 Dec 2015 00:35:53 GMT
                    Content-Type: text/plain
                    Content-Length: 278 
                    Connection: keep-alive
                    x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
                    Accept-Ranges: bytes
                    ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
                    Last-Modified: Sat, 12 Dec 2015 00:35:53 GMT
x-oss-object-type: Normal

<?xml version="1.0" encoding="UTF-8"?>
<Tagging>
<TagSet>
<Tag>
<Key>k1</Key>
<Value>v1</Value>
</Tag>
<Tag>
<Key>k2</Key>
<Value>v2</Value>
</Tag>
<Tag>
<Key>k3</Key>
<Value>v3</Value>
</Tag>
</TagSet>
</Tagging>'''

    return request_text, response_text

class TestObject(unittests.common.OssTestCase):
    @patch('oss2.Session.do_request')
    def test_head(self, do_request):
        request_text = '''HEAD /apbmntxqtvxjzini HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:35:55 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:Q05CWxpclrtNnUWHY5wS10fhFk0='''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:55 GMT
Content-Type: application/octet-stream
Content-Length: 10
Connection: keep-alive
x-oss-request-id: 566B6BEBD4C05B21E97261B0
Accept-Ranges: bytes
ETag: "0CF031A5EB9351746195B20B86FD3F68"
Last-Modified: Sat, 12 Dec 2015 00:35:54 GMT
x-oss-object-type: Normal'''

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().head_object('apbmntxqtvxjzini')

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.content_length, 10)
        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BEBD4C05B21E97261B0')
        self.assertEqual(result.object_type, 'Normal')
        self.assertEqual(result.content_type, 'application/octet-stream')
        self.assertEqual(result.etag, '0CF031A5EB9351746195B20B86FD3F68')
        self.assertEqual(result.last_modified, 1449880554)

    @patch('oss2.Session.do_request')
    def test_object_exists_true(self, do_request):
        request_text = '''GET /sbowspxjhmccpmesjqcwagfw?objectMeta HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:37:17 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:wopWcmMd/70eNKYOc9M6ZA21yY8='''

        response_text = '''HTTP/1.1 200 OK
x-oss-request-id: 566B6C3D010B7A4314D2253D
Date: Sat, 12 Dec 2015 00:37:17 GMT
ETag: "5B3C1A2E053D763E1B002CC607C5A0FE"
Last-Modified: Sat, 12 Dec 2015 00:37:17 GMT
Content-Length: 344606
Connection: keep-alive
Server: AliyunOSS'''

        req_info = unittests.common.mock_response(do_request, response_text)

        self.assertTrue(unittests.common.bucket().object_exists('sbowspxjhmccpmesjqcwagfw'))
        self.assertRequest(req_info, request_text)

    @patch('oss2.Session.do_request')
    def test_object_exists_false(self, do_request):
        request_text = '''GET /sbowspxjhmccpmesjqcwagfw?objectMeta HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:37:17 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:wopWcmMd/70eNKYOc9M6ZA21yY8='''     

        response_text = '''HTTP/1.1 404 Not Found
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:37:17 GMT
Content-Type: application/xml
Content-Length: 287
Connection: keep-alive
x-oss-request-id: 566B6C3D6086505A0CFF0F68

<?xml version="1.0" encoding="UTF-8"?>
<Error>
  <Code>NoSuchKey</Code>
  <Message>The specified key does not exist.</Message>
  <RequestId>566B6C3D6086505A0CFF0F68</RequestId>
  <HostId>ming-oss-share.oss-cn-hangzhou.aliyuncs.com</HostId>
  <Key>sbowspxjhmccpmesjqcwagfw</Key>
</Error>'''

        req_info = unittests.common.mock_response(do_request, response_text)
        self.assertTrue(not unittests.common.bucket().object_exists('sbowspxjhmccpmesjqcwagfw'))
        self.assertRequest(req_info, request_text)
    
    @patch('oss2.Session.do_request')
    def test_object_exists_exception(self, do_request):
        request_text = '''GET /sbowspxjhmccpmesjqcwagfw?objectMeta HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:37:17 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:wopWcmMd/70eNKYOc9M6ZA21yY8='''     

        response_text = '''HTTP/1.1 404 Not Found
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:37:17 GMT
Content-Type: application/xml
Content-Length: 287
Connection: keep-alive
x-oss-request-id: 566B6C3D6086505A0CFF0F68

<?xml version="1.0" encoding="UTF-8"?>
<Error>
  <Code>NoSuchBucket</Code>
  <Message>The specified bucket does not exist.</Message>
  <RequestId>566B6C3D6086505A0CFF0F68</RequestId>
  <HostId>ming-oss-share.oss-cn-hangzhou.aliyuncs.com</HostId>
  <Bucket>ming-oss-share</Bucket>
</Error>'''

        unittests.common.mock_response(do_request, response_text)
        self.assertRaises(oss2.exceptions.NoSuchBucket, unittests.common.bucket().object_exists, 'sbowspxjhmccpmesjqcwagfw')
    
    @patch('oss2.Session.do_request')
    def test_get_object_meta(self, do_request):
        request_text = '''GET /sbowspxjhmccpmesjqcwagfw?objectMeta HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
date: Sat, 12 Dec 2015 00:37:17 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:wopWcmMd/70eNKYOc9M6ZA21yY8='''

        response_text = '''HTTP/1.1 200 OK
x-oss-request-id: 566B6C3D010B7A4314D2253D
Date: Sat, 12 Dec 2015 00:37:17 GMT
ETag: "5B3C1A2E053D763E1B002CC607C5A0FE"
Last-Modified: Sat, 12 Dec 2015 00:37:17 GMT
Content-Length: 344606
Connection: keep-alive
Server: AliyunOSS'''

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().get_object_meta('sbowspxjhmccpmesjqcwagfw')
        
        self.assertRequest(req_info, request_text)

        self.assertEqual(result.last_modified, 1449880637)
        self.assertEqual(result.content_length, 344606)
        self.assertEqual(result.etag, '5B3C1A2E053D763E1B002CC607C5A0FE')

    @patch('oss2.Session.do_request')
    def test_get(self, do_request):
        content = unittests.common.random_bytes(1023)

        request_text, response_text = make_get_object(content)

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().get_object('sjbhlsgsbecvlpbf')

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.read(), content)
        self.assertEqual(result.content_length, len(content))
        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.object_type, 'Normal')
        self.assertEqual(result.content_type, 'text/plain')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')
        self.assertEqual(result.last_modified, 1449880553)

    @patch('oss2.Session.do_request')
    def test_get_with_progress(self, do_request):
        content = unittests.common.random_bytes(1024 * 1024 + 1)

        request_text, response_text = make_get_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        self.previous = -1
        result = unittests.common.bucket().get_object('sjbhlsgsbecvlpbf', progress_callback=self.progress_callback)

        self.assertRequest(req_info, request_text)

        content_read = unittests.common.read_file(result)

        self.assertEqual(self.previous, len(content))
        self.assertEqual(len(content_read), len(content))
        self.assertEqual(oss2.to_bytes(content_read), content)

    @patch('oss2.Session.do_request')
    def test_get_to_file(self, do_request):
        content = unittests.common.random_bytes(1023)

        request_text, response_text = make_get_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        filename = self.tempname()

        result = unittests.common.bucket().get_object_to_file('sjbhlsgsbecvlpbf', filename)

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.content_length, len(content))
        self.assertEqual(os.path.getsize(filename), len(content))

        with open(filename, 'rb') as f:
            self.assertEqual(content, oss2.to_bytes(f.read()))

    @patch('oss2.Session.do_request')
    def test_get_to_file_with_progress(self, do_request):
        size = 1024 * 1024 + 1
        content = unittests.common.random_bytes(size)

        request_text, response_text = make_get_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        filename = self.tempname()

        self.previous = -1
        unittests.common.bucket().get_object_to_file('sjbhlsgsbecvlpbf', filename, progress_callback=self.progress_callback)

        self.assertRequest(req_info, request_text)

        self.assertEqual(self.previous, size)
        self.assertEqual(os.path.getsize(filename), size)
        with open(filename, 'rb') as f:
            self.assertEqual(oss2.to_bytes(content), f.read())

    @patch('oss2.Session.do_request')
    def test_put_result(self, do_request):
        content = b'dummy content'
        request_text, response_text = make_put_object(content)

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().put_object('sjbhlsgsbecvlpbf.txt', content)

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')

    @patch('oss2.Session.do_request')
    def test_put_bytes(self, do_request):
        content = unittests.common.random_bytes(1024 * 1024 - 1)

        request_text, response_text = make_put_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        unittests.common.bucket().put_object('sjbhlsgsbecvlpbf.txt', content)

        self.assertRequest(req_info, request_text)

    @patch('oss2.Session.do_request')
    def test_put_bytes_with_progress(self, do_request):
        self.previous = -1

        content = unittests.common.random_bytes(1024 * 1024 - 1)

        request_text, response_text = make_put_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        unittests.common.bucket().put_object('sjbhlsgsbecvlpbf.txt', content, progress_callback=self.progress_callback)

        self.assertRequest(req_info, request_text)
        self.assertEqual(self.previous, len(content))

    @patch('oss2.Session.do_request')
    def test_put_from_file(self, do_request):
        size = 512 * 2 - 1
        content = unittests.common.random_bytes(size)
        filename = self.make_tempfile(content)

        request_text, response_text = make_put_object(content)
        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().put_object_from_file('sjbhlsgsbecvlpbf.txt', filename)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')

    @patch('oss2.Session.do_request')
    def test_put_without_crc_in_response(self, do_request):
        content = b'dummy content'
        request_text = '''PUT /sjbhlsgsbecvlpbf.txt HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Type: text/plain
Content-Length: {0}
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:W6whAowN4aImQ0dfbMHyFfD0t1g=
Accept: */*

{1}'''.format(len(content), oss2.to_string(content))
        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"'''

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().put_object('sjbhlsgsbecvlpbf.txt', content)

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')

    @patch('oss2.Session.do_request')
    def test_append(self, do_request):
        size = 8192 * 2 - 1
        content = unittests.common.random_bytes(size)

        request_text, response_text = make_append_object(0, content)
        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().append_object('sjbhlsgsbecvlpbf', 0, content)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.status, 200)
        self.assertEqual(result.next_position, size)
        self.assertEqual(result.etag, '24F7FA10676D816E0D6C6B5600000000')
        self.assertEqual(result.crc, unittests.common.calc_crc(content))

    @patch('oss2.Session.do_request')
    def test_append_with_progress(self, do_request):
        size = 1024 * 1024
        content = unittests.common.random_bytes(size)

        request_text, response_text = make_append_object(0, content)
        req_info = unittests.common.mock_response(do_request, response_text)

        self.previous = -1

        result = unittests.common.bucket().append_object('sjbhlsgsbecvlpbf', 0, content, progress_callback=self.progress_callback)

        self.assertRequest(req_info, request_text)
        self.assertEqual(self.previous, size)
        self.assertEqual(result.next_position, size)

    @patch('oss2.Session.do_request')
    def test_append_without_crc_in_response(self, do_request):
        size = 8192
        position = 0
        content = unittests.common.random_bytes(size)
        request_text = '''POST /sjbhlsgsbecvlpbf?position={0}&append= HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: {1}
date: Sat, 12 Dec 2015 00:36:29 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:1njpxsTivMNvTdfYolCUefRInVY=

{2}'''.format(position, len(content), oss2.to_string(content))

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:36:29 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6C0D1790CF586F72240B
ETag: "24F7FA10676D816E0D6C6B5600000000"
x-oss-next-append-position: {0}'''.format(position + len(content), unittests.common.calc_crc(content))
        
        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().append_object('sjbhlsgsbecvlpbf', position, content, init_crc=0)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.status, 200)
        self.assertEqual(result.next_position, size)
        self.assertEqual(result.etag, '24F7FA10676D816E0D6C6B5600000000')
        
    @patch('oss2.Session.do_request')
    def test_delete(self, do_request):
        request_text = '''DELETE /sjbhlsgsbecvlpbf HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 0
date: Sat, 12 Dec 2015 00:36:29 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:AC830VOm7dDnv+CVpTaui6gh5xc='''

        response_text = '''HTTP/1.1 204 No Content
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:36:29 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6C0D8CDE4E975D730BEF'''

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().delete_object('sjbhlsgsbecvlpbf')

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.request_id, '566B6C0D8CDE4E975D730BEF')
        self.assertEqual(result.status, 204)

    def test_batch_delete_empty(self):
        self.assertRaises(oss2.exceptions.ClientError, unittests.common.bucket().batch_delete_objects, [])

    @patch('oss2.Session.do_request')
    def test_batch_delete(self, do_request):
        request_text = '''POST /?delete=&encoding-type=url HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 100
Content-MD5: zsbG45tEj+StFBFghUllvw==
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:tc4g/qgaHwQ+CoI828v2zFCHj2E=

<Delete><Quiet>false</Quiet><Object><Key>hello</Key></Object><Object><Key>world</Key></Object></Delete>'''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Type: application/xml
Content-Length: 383
Connection: keep-alive
x-oss-request-id: 566B6BE9229E6BA1F6F538DE

<?xml version="1.0" encoding="UTF-8"?>
<DeleteResult>
<EncodingType>url</EncodingType>
<Deleted>
    <Key>hello</Key>
</Deleted>
<Deleted>
    <Key>world</Key>
</Deleted>
</DeleteResult>'''
        req_info = unittests.common.mock_response(do_request, response_text)

        key_list = ['hello', 'world']

        result = unittests.common.bucket().batch_delete_objects(key_list)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.deleted_keys, list(oss2.to_string(key) for key in key_list))

    @patch('oss2.Session.do_request')
    def test_copy_object(self, do_request):
        request_text = '''PUT /zyfpyqqqxjthdwxkhypziizm.js HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Content-Length: 0
x-oss-copy-source: /ming-oss-share/zyfpyqqqxjthdwxkhypziizm.js
x-oss-meta-category: novel
Content-Type: text/plain
Connection: keep-alive
date: Sat, 12 Dec 2015 00:37:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:azW764vWaOVYhJLdhw4sEntNYP4=
Accept: */*'''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:37:53 GMT
Content-Type: application/xml
Content-Length: 184
Connection: keep-alive
x-oss-request-id: 566B6C611BA604C27DD51F8F
ETag: "164F32EF262006C5EE6C8D1AA30DD2CD"

<?xml version="1.0" encoding="UTF-8"?>
<CopyObjectResult>
  <ETag>"164F32EF262006C5EE6C8D1AA30DD2CD"</ETag>
  <LastModified>2015-12-12T00:37:53.000Z</LastModified>
</CopyObjectResult>'''

        req_info = unittests.common.mock_response(do_request, response_text)

        in_headers = {'Content-Type': 'text/plain', 'x-oss-meta-category': 'novel'}
        result = unittests.common.bucket().update_object_meta('zyfpyqqqxjthdwxkhypziizm.js', in_headers)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.request_id, '566B6C611BA604C27DD51F8F')
        self.assertEqual(result.etag, '164F32EF262006C5EE6C8D1AA30DD2CD')

    @patch('oss2.Session.do_request')
    def test_put_acl(self, do_request):
        req_info = unittests.common.RequestInfo()

        do_request.auto_spec = True
        do_request.side_effect = partial(unittests.common.do4put, req_info=req_info)

        for acl, expected in [(oss2.OBJECT_ACL_PRIVATE, 'private'),
                              (oss2.OBJECT_ACL_PUBLIC_READ, 'public-read'),
                              (oss2.OBJECT_ACL_PUBLIC_READ_WRITE, 'public-read-write'),
                              (oss2.OBJECT_ACL_DEFAULT, 'default')]:
            unittests.common.bucket().put_object_acl('fake-key', acl)
            self.assertEqual(req_info.req.headers['x-oss-object-acl'], expected)

    @patch('oss2.Session.do_request')
    def test_get_acl(self, do_request):
        template = '''<?xml version="1.0" encoding="UTF-8"?>
        <AccessControlPolicy>
          <Owner>
            <ID>1047205513514293</ID>
            <DisplayName>1047205513514293</DisplayName>
          </Owner>
          <AccessControlList>
            <Grant>{0}</Grant>
          </AccessControlList>
        </AccessControlPolicy>
        '''

        for acl, expected in [(oss2.OBJECT_ACL_PRIVATE, 'private'),
                              (oss2.OBJECT_ACL_PUBLIC_READ, 'public-read'),
                              (oss2.OBJECT_ACL_PUBLIC_READ_WRITE, 'public-read-write'),
                              (oss2.OBJECT_ACL_DEFAULT, 'default')]:
            do_request.auto_spec = True
            do_request.side_effect = partial(unittests.common.do4body, body=template.format(acl), content_type='application/xml')

            result = unittests.common.bucket().get_object_acl('fake-key')
            self.assertEqual(result.acl, expected)
    
    @patch('oss2.Session.do_request')
    def test_put_symlink(self, do_request):
        request_text = '''PUT /sjbhlsgsbecvlpbf?symlink= HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 0
User-Agent: aliyun-sdk-python/2.3.0(Windows/7/;3.3.3)
x-oss-symlink-target: bcvzkwznomy
x-oss-meta-key1: value1
x-oss-meta-key2: value2
date: Wed, 22 Mar 2017 03:15:15 GMT
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:AC830VOm7dDnv+CVpTaui6gh5xc='''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Wed, 22 Mar 2017 03:15:20 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6C0D8CDE4E975D730BEF
ETag: "B070B9DEB1655BE905777D6DC856E6F1"
x-oss-hash-crc64ecma: 0
x-oss-server-time: 19'''

        req_info = unittests.common.mock_response(do_request, response_text)

        headers = {'x-oss-meta-key1': 'value1', 'x-oss-meta-key2': 'value2'}
        result = unittests.common.bucket().put_symlink('bcvzkwznomy', 'sjbhlsgsbecvlpbf', headers)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.request_id, '566B6C0D8CDE4E975D730BEF')
        self.assertEqual(result.status, 200)
        
    @patch('oss2.Session.do_request')
    def test_get_symlink(self, do_request):
        request_text = '''GET /sjbhlsgsbecvlpbf?symlink= HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Accept: */*
User-Agent: aliyun-sdk-python/2.3.0(Windows/7/;3.3.3)
date: Wed, 22 Mar 2017 03:14:31 GMT
authorization: OSS ZCDmm7TPZKHtx77j:AC830VOm7dDnv+CVpTaui6gh5xc='''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Wed, 22 Mar 2017 03:14:36 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6C0D8CDE4E975D730BEF
Last-Modified: Wed, 22 Mar 2017 03:14:31 GMT
ETag: "0D9980D049C9256C927F8A46BC1BADCF"
x-oss-symlink-target: bcvzkwznomy
x-oss-server-time: 39'''

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().get_symlink('sjbhlsgsbecvlpbf')

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.request_id, '566B6C0D8CDE4E975D730BEF')
        self.assertEqual(result.status, 200)
        self.assertEqual(result.target_key, 'bcvzkwznomy')

    @patch('oss2.Session.do_request')
    def test_crypto_get(self, do_request):
        content = unittests.common.random_bytes(1023)

        key = random_string(10)
        provider = oss2.RsaProvider(key_pair=key_pair)
        content_crypto_material = provider.create_content_material()

        request_text, response_text = make_get_encrypted_object(key, content, content_crypto_material)
        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket(provider).get_object(key)

        encrypted_key = utils.b64encode_as_string(content_crypto_material.encrypted_key)
        encrypted_iv = utils.b64encode_as_string(content_crypto_material.encrypted_iv)
        wrap_alg = content_crypto_material.wrap_alg
        cek_alg = content_crypto_material.cek_alg

        self.assertRequest(req_info, request_text)
        self.assertEqual(int(result.headers['x-oss-meta-unencrypted-content-length']), len(content))
        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.object_type, 'Normal')
        self.assertEqual(result.content_type, 'text/plain')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')
        self.assertEqual(result.last_modified, 1449880553)
        self.assertEqual(result.headers['x-oss-meta-client-side-encryption-key'], encrypted_key)
        self.assertEqual(result.headers['x-oss-meta-client-side-encryption-start'], encrypted_iv)
        self.assertEqual(result.headers['x-oss-meta-client-side-encryption-cek-alg'], cek_alg)
        self.assertEqual(result.headers['x-oss-meta-client-side-encryption-wrap-alg'], wrap_alg)
        self.assertEqual(result.read(), content)

    @patch('oss2.Session.do_request')
    def test_crypto_get_with_inconsistent_crypto_meta(self, do_request):
        content = unittests.common.random_bytes(1023)

        key = random_bytes(10)
        provider = oss2.RsaProvider(key_pair=key_pair)
        content_crypto_material = provider.create_content_material()

        request_text, response_text = make_get_encrypted_object(key, content, content_crypto_material,
                                                                invalid_cek_alg='invald_cek_alg')
        req_info = unittests.common.mock_response(do_request, response_text)

        self.assertRaises(oss2.exceptions.InconsistentError, unittests.common.bucket(provider).get_object, key)

    @patch('oss2.Session.do_request')
    def test_crypto_get_compact(self, do_request):
        example_path = "tests/example.jpg"
        with open(example_path, 'rb') as f:
            content = f.read()

        for compact_sdk in ["cpp", "go"]:
            if compact_sdk == "cpp":
                encrypted_example_path = "tests/encrypted_cpp_example.jpg"
                encrypted_example_meta_path = "tests/encrypted_cpp_example_meta.json"
            else:
                encrypted_example_path = "tests/encrypted_go_example.jpg"
                encrypted_example_meta_path = "tests/encrypted_go_example_meta.json"

            with open(encrypted_example_path, 'rb') as f:
                encrypted_content = f.read()

            with open(encrypted_example_meta_path, 'r') as f:
                meta = json.loads(f.read())

            key = random_string(10)
            provider = oss2.RsaProvider(key_pair=key_pair_compact)

            request_text, response_text = make_get_encrypted_object_compact(key, encrypted_content, meta)

            req_info = unittests.common.mock_response(do_request, response_text)
            result = unittests.common.bucket(provider).get_object(key)

            self.assertRequest(req_info, request_text)
            self.assertEqual(result.read(), content)

    @patch('oss2.Session.do_request')
    def test_crypto_get_compact_deprecated_rsa(self, do_request):
        utils.silently_remove('./rsa-test.public_key.pem')
        utils.silently_remove('./rsa-test.private_key.pem')

        with open("./rsa-test.private_key.pem", 'wb') as f:
            f.write(oss2.to_bytes(private_key_compact))

        with open("./rsa-test.public_key.pem", 'wb') as f:
            f.write(oss2.to_bytes(public_key_compact))

        content = b'a' * 1024 * 1024
        encrypted_rsa_path = "tests/deprecated_encrypted_1MB_a_rsa"
        encrypted_meta_rsa_path = "tests/deprecated_encrypted_1MB_a_meta_rsa.json"

        with open(encrypted_rsa_path, 'rb') as f:
            encrypted_content = f.read()

        with open(encrypted_meta_rsa_path, 'r') as f:
            meta = json.loads(f.read())

        key = random_string(10)
        provider = oss2.LocalRsaProvider(dir='./', key='rsa-test')

        request_text, response_text = make_get_encrypted_object_compact_deprecated(key, encrypted_content, meta)

        req_info = unittests.common.mock_response(do_request, response_text)
        result = unittests.common.bucket(provider).get_object(key)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.read(), content)

        utils.silently_remove('./rsa-test.public_key.pem')
        utils.silently_remove('./rsa-test.private_key.pem')

    @patch('oss2.Session.do_request')
    def test_crypto_get_compact_deprecated_kms(self, do_request):
        content = b'a' * 1024 * 1024
        encrypted_kms_path = "tests/deprecated_encrypted_1MB_a_kms"
        encrypted_meta_kms_path = "tests/deprecated_encrypted_1MB_a_meta_kms.json"

        with open(encrypted_kms_path, 'rb') as f:
            encrypted_content = f.read()

        with open(encrypted_meta_kms_path, 'r') as f:
            meta = json.loads(f.read())

        key = random_string(10)
        provider = oss2.AliKMSProvider(OSS_ID, OSS_SECRET, OSS_REGION, OSS_CMK)

        request_text, response_text = make_get_encrypted_object_compact_deprecated(key, encrypted_content, meta)
        plain_key = utils.b64decode_from_string(meta['base64-plain-key'])
        plain_start = meta['plain-start']

        req_info = unittests.common.mock_response(do_request, response_text)
        with patch('oss2.AliKMSProvider.decrypt_encrypted_key', return_value=plain_key):
            with patch('oss2.AliKMSProvider.decrypt_encrypted_iv', return_value=plain_start):
                result = unittests.common.bucket(provider).get_object(key)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.read(), content)

    @patch('oss2.Session.do_request')
    def test_crypto_put(self, do_request):
        content = unittests.common.random_bytes(1023)

        key = random_string(10)
        provider = oss2.RsaProvider(key_pair=key_pair)
        plain_key = provider.get_key()
        plain_iv = provider.get_iv()

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()

        request_text, response_text = make_put_encrypted_object(key, content, content_crypto_material)
        req_info = unittests.common.mock_response(do_request, response_text)

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                result = unittests.common.bucket(provider).put_object(key, content, headers={
                    'content-md5': oss2.utils.md5_string(content), 'content-length': str(len(content))})

        # 这个地方两次加密形成的加密key和向量会不一致，所以将这行注释掉
        # self.assertRequest(req_info, request_text)
        self.assertEqual(result.status, 200)
        self.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
        self.assertEqual(result.etag, 'D80CF0E5BE2436514894D64B2BCFB2AE')


    @patch('oss2.Session.do_request')
    def test_get_object_tagging(self, do_request):

        request_text, response_text = make_get_object_tagging()

        req_info = unittests.common.mock_response(do_request, response_text)

        result = unittests.common.bucket().get_object_tagging('sjbhlsgsbecvlpbf')

        req_info = unittests.common.mock_response(do_request, response_text)

        self.assertEqual(3, result.tag_set.len())
        self.assertEqual('v1', result.tag_set.tagging_rule['k1'])
        self.assertEqual('v2', result.tag_set.tagging_rule['k2'])
        self.assertEqual('v3', result.tag_set.tagging_rule['k3'])


    # for ci
    def test_oss_utils_negative(self):
        try:
            oss2.utils.makedir_p('/')
            self.assertTrue(False)
        except:
            pass
        
        try:
            oss2.utils.silently_remove('/')
            self.assertTrue(False)
        except:
            pass
        
        try:
            oss2.utils.force_rename('/', '/')
            self.assertTrue(False)
        except:
            pass
        
        oss2.utils.makedir_p('xyz')
        oss2.utils.makedir_p('zyz')
        try:
            oss2.utils.force_rename('xyz', 'zyx')
            self.assertTrue(False)
        except:
            pass
            

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import os
import shutil

import oss2


# 以下代码展示了基本的文件上传、下载、罗列、删除用法。


# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
#
# 以杭州区域为例，Endpoint可以是：
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
# 分别以HTTP、HTTPS协议访问。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)


# 上传一段字符串。Object名是motto.txt，内容是一段名言。
bucket.put_object('motto.txt', 'Never give up. - Jack Ma')

# 获取Object的metadata
object_meta = bucket.get_object_meta('你的对象名')
print('last modified: ' + object_meta.last_modified)
print('etag: ' + object_meta.etag)
print('size: ' + object_meta.content_length)

# 下载到本地文件
bucket.get_object_to_file('motto.txt', '本地文件名.txt')


# 把刚刚上传的Object下载到本地文件 “座右铭.txt” 中
# 因为get_object()方法返回的是一个file-like object，所以我们可以直接用shutil.copyfileobj()做拷贝
with open(oss2.to_unicode('本地座右铭.txt'), 'wb') as f:
    shutil.copyfileobj(bucket.get_object('motto.txt'), f)


# 把本地文件 “座右铭.txt” 上传到OSS，新的Object叫做 “我的座右铭.txt”
# 注意到，这次put_object()的第二个参数是file object；而上次上传是一个字符串。
# put_object()能够识别不同的参数类型
with open(oss2.to_unicode('本地座右铭.txt'), 'rb') as f:
    bucket.put_object('云上座右铭.txt', f)


# 上面两行代码，也可以用下面的一行代码来实现
bucket.put_object_from_file('云上座右铭.txt', '本地座右铭.txt')


# 列举Bucket下10个Object，并打印它们的最后修改时间、文件名
for i, object_info in enumerate(oss2.ObjectIterator(bucket)):
    print("{0} {1}".format(object_info.last_modified, object_info.key))

    if i >= 9:
        break


# 删除名为motto.txt的Object
bucket.delete_object('motto.txt')

# 也可以批量删除
# 注意：重复删除motto.txt，并不会报错
bucket.batch_delete_objects(['motto.txt', '云上座右铭.txt'])


# 确认Object已经被删除了
assert not bucket.object_exists('motto.txt')


# 获取不存在的文件会抛出oss2.exceptions.NoSuchKey异常
try:
    bucket.get_object('云上座右铭.txt')
except oss2.exceptions.NoSuchKey as e:
    print(u'已经被删除了：request_id={0}'.format(e.request_id))
else:
    assert False

# 清除本地文件
os.remove(u'本地文件名.txt')
os.remove(u'本地座右铭.txt')



# -*- coding: utf-8 -*-

"""
oss2.http
~~~~~~~~

这个模块包含了HTTP Adapters。尽管OSS Python SDK内部使用requests库进行HTTP通信，但是对使用者是透明的。
该模块中的 `Session` 、 `Request` 、`Response` 对requests的对应的类做了简单的封装。
"""

import platform

import requests
from requests.structures import CaseInsensitiveDict

from . import __version__, defaults
from .compat import to_bytes
from .exceptions import RequestError
from .utils import file_object_remaining_bytes, SizedFileAdapter

import logging

USER_AGENT = 'aliyun-sdk-python/{0}({1}/{2}/{3};{4})'.format(
    __version__, platform.system(), platform.release(), platform.machine(), platform.python_version())

logger = logging.getLogger(__name__)


class Session(object):
    """属于同一个Session的请求共享一组连接池，如有可能也会重用HTTP连接。"""

    def __init__(self):
        self.session = requests.Session()

        psize = defaults.connection_pool_size
        self.session.mount('http://', requests.adapters.HTTPAdapter(pool_connections=psize, pool_maxsize=psize))
        self.session.mount('https://', requests.adapters.HTTPAdapter(pool_connections=psize, pool_maxsize=psize))

    def do_request(self, req, timeout):
        try:
            logger.debug("Send request, method: {0}, url: {1}, params: {2}, headers: {3}, timeout: {4}, proxies: {5}".format(
                req.method, req.url, req.params, req.headers, timeout, req.proxies))
            return Response(self.session.request(req.method, req.url,
                                                 data=req.data,
                                                 params=req.params,
                                                 headers=req.headers,
                                                 stream=True,
                                                 timeout=timeout,
                                                 proxies=req.proxies))
        except requests.RequestException as e:
            raise RequestError(e)


class Request(object):
    def __init__(self, method, url,
                 data=None,
                 params=None,
                 headers=None,
                 app_name='',
                 proxies=None):
        self.method = method
        self.url = url
        self.data = _convert_request_body(data)
        self.params = params or {}
        self.proxies = proxies

        if not isinstance(headers, CaseInsensitiveDict):
            self.headers = CaseInsensitiveDict(headers)
        else:
            self.headers = headers

        # tell requests not to add 'Accept-Encoding: gzip, deflate' by default
        if 'Accept-Encoding' not in self.headers:
            self.headers['Accept-Encoding'] = None

        if 'User-Agent' not in self.headers:
            if app_name:
                self.headers['User-Agent'] = USER_AGENT + '/' + app_name
            else:
                self.headers['User-Agent'] = USER_AGENT

        logger.debug("Init request, method: {0}, url: {1}, params: {2}, headers: {3}".format(method, url, params,
                                                                                             headers))


_CHUNK_SIZE = 8 * 1024


class Response(object):
    def __init__(self, response):
        self.response = response
        self.status = response.status_code
        self.headers = response.headers
        self.request_id = response.headers.get('x-oss-request-id', '')

        # When a response contains no body, iter_content() cannot
        # be run twice (requests.exceptions.StreamConsumedError will be raised).
        # For details of the issue, please see issue #82
        #
        # To work around this issue, we simply return b'' when everything has been read.
        #
        # Note you cannot use self.response.raw.read() to implement self.read(), because
        # raw.read() does not uncompress response body when the encoding is gzip etc., and
        # we try to avoid depends on details of self.response.raw.
        self.__all_read = False

        logger.debug("Get response headers, req-id:{0}, status: {1}, headers: {2}".format(self.request_id, self.status,
                                                                                          self.headers))

    def read(self, amt=None):
        if self.__all_read:
            return b''

        if amt is None:
            content_list = []
            for chunk in self.response.iter_content(_CHUNK_SIZE):
                content_list.append(chunk)
            content = b''.join(content_list)

            self.__all_read = True
            return content
        else:
            try:
                return next(self.response.iter_content(amt))
            except StopIteration:
                self.__all_read = True
                return b''

    def __iter__(self):
        return self.response.iter_content(_CHUNK_SIZE)


# requests对于具有fileno()方法的file object，会用fileno()的返回值作为Content-Length。
# 这对于已经读取了部分内容，或执行了seek()的file object是不正确的。
#
# _convert_request_body()对于支持seek()和tell() file object，确保是从
# 当前位置读取，且只读取当前位置到文件结束的内容。
def _convert_request_body(data):
    data = to_bytes(data)

    if hasattr(data, '__len__'):
        return data

    if hasattr(data, 'seek') and hasattr(data, 'tell'):
        return SizedFileAdapter(data, file_object_remaining_bytes(data))

    return data




import os
import oss2
import json

# 以下代码展示了bucket_policy相关API的用法，
# 具体policy书写规则参考官网文档说明

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
#
# 以杭州区域为例，Endpoint可以是：
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

# 创建policy_text
policy=dict()
policy["Version"] = "1"
policy["Statement"] = []
statement = dict()
statement["Action"] = ["oss:PutObject"]
statement["Effect"] = "Allow"
statement["Resource"] = ["acs:oss:*:*:*/*"]
policy["Statement"].append(statement)
policy_text = json.dumps(policy)

# Put bolicy_text
print("Put policy text : ", policy_text)
bucket.put_bucket_policy(policy_text)

# Get bucket Policy
result = bucket.get_bucket_policy()
policy_json = json.loads(result.policy) 
print("Get policy text: ", policy_json)

# 校验返回的policy
assert len(policy["Statement"]) == len(policy_json["Statement"])
assert policy["Version"] == policy_json["Version"]
policy_resource = policy["Statement"][0]["Resource"][0]
policy_json_resource = policy_json["Statement"][0]["Resource"][0]
assert policy_resource == policy_json_resource

# 删除policy
result = bucket.delete_bucket_policy()
assert int(result.status)//100 == 2


# -*- coding: utf-8 -*-

from .common import *
from oss2.compat import to_bytes


class TestPutObjectChunked(OssTestCase):
    def test_put_object_chunked(self):
        class FakeFileObj(object):
            def __init__(self, data, size):
                self.data = to_bytes(data)
                self.offset = 0
                self.size = size

            def read(self, amt=None):
                if self.offset >= self.size:
                    return to_bytes('')

                if amt is None or amt < 0:
                    bytes_to_read = self.size - self.offset
                else:
                    bytes_to_read = min(amt, self.size - self.offset)

                content = self.data[self.offset:self.offset + bytes_to_read]

                self.offset += bytes_to_read

                return content

        object_name = 'test-put-file-like-object-chunked'

        count = 1
        while count <= 1500:
            try:
                count += 1

                cnt = random.randint(count, 102400)
                data = FakeFileObj(b'a' * cnt, count)
                self.bucket.put_object(object_name + str(count) + '.txt', data)
                cnt = random.randint(count, 102400)
                data = FakeFileObj(b'a' * cnt, cnt)
                self.bucket.put_object(object_name + str(count) + '11.txt', data)
            except oss2.exceptions.ServerError as e:
                if e.code == 'BadRequest':
                    raise
            except Exception:
                continue


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import os

import oss2
import sys
import struct

from functools import partial
from oss2 import to_string
from mock import patch
from oss2 import xml_utils
from oss2 import utils
from oss2.exceptions import SelectOperationClientError
from oss2.exceptions import InconsistentError
from oss2.exceptions import SelectOperationFailed
from oss2.exceptions import ClientError

from unittests.common import *



class ContiniousFrame(object):
    _CONTINIOUS_FRAME_TYPE=8388612
    _DATA_FRAME_TYPE = 8388609
    _END_FRAME_TYPE = 8388613
    _META_END_FRAME_TYPE = 8388614
    _JSON_META_END_FRAME_TYPE = 8388615
    _HEAD_CHECK_SUM = 0
    def __init__(self, offset, payload = None, payload_length = 8, frame_type = 8388612):
         self.offset = offset
         self.payload_length = payload_length
         self.frame_type =  frame_type | 0x1000000
         if (payload is None):
            self.payload = struct.pack('!Q', offset)
         else:
            self.payload = struct.pack('!Q', offset) + payload
    
    def header_str(self):
         return struct.pack('!III', self.frame_type, self.payload_length, ContiniousFrame._HEAD_CHECK_SUM)

    def to_bytes(self):
         header = self.header_str()
         crc32 = utils.Crc32()
         crc32.update(self.payload)
         checksum_calc = struct.pack('!I', crc32.crc)
         return header + self.payload + checksum_calc
    
class DataFrame(ContiniousFrame):
    
     def __init__(self, offset, data):
         super(DataFrame, self).__init__(offset, data, len(data) + 8, ContiniousFrame._DATA_FRAME_TYPE)
         self.data = data
    
class EndFrame(ContiniousFrame):
    
    def __init__(self, offset, scanned_size, status, error = b''):
        super(EndFrame, self).__init__(offset, struct.pack('!QI', scanned_size, status) + error, len(error) + 20, ContiniousFrame._END_FRAME_TYPE)
        self.scanned_size = scanned_size
        self.status = status
        self.error = error

class EndMetaFrame(ContiniousFrame):
    def __init__(self, offset, scannedsize, status, splits, rows, cols, error):
        super(EndMetaFrame, self).__init__(offset, struct.pack('!QIIQI', scannedsize, status, splits, rows, cols) + error, 36 + len(error), ContiniousFrame._META_END_FRAME_TYPE)
        self.scanned_size = scannedsize
        self.status = status
        self.splits = splits
        self.rows = rows
        self.cols = cols

class EndJsonMetaFrame(ContiniousFrame):
    def __init__(self, offset, scannedsize, status, splits, rows, error):
        super(EndJsonMetaFrame, self).__init__(offset, struct.pack('!QIIQ', scannedsize, status, splits, rows) + error, 32 + len(error), ContiniousFrame._JSON_META_END_FRAME_TYPE)
        self.scanned_size = scannedsize
        self.status = status
        self.splits = splits
        self.rows = rows

def generate_data(resp_content, output_raw, error = b'', status = 206, simulate_bad_crc = False):
    if output_raw:
        return to_string(resp_content)
    else:
        continiousFrame = ContiniousFrame(100)
        dataFrame = DataFrame(len(resp_content), resp_content)
        frameStr = dataFrame.to_bytes()
        if (simulate_bad_crc):
            frameStr = frameStr[0:len(frameStr)-2] + b'X'  # 'X' is the character that does not exist in the content

        endFrame = EndFrame(len(resp_content), len(resp_content), status, error)

        return continiousFrame.to_bytes() + frameStr + endFrame.to_bytes()

def generate_head_data(scanned_size, splits, rows, cols = None, error = b'', status = 200):
    continiousFrame = ContiniousFrame(100)
    if cols is not None:
        endFrame = EndMetaFrame(scanned_size, scanned_size, status, splits, rows, cols, error)
    else:
        endFrame = EndJsonMetaFrame(scanned_size, scanned_size, status, splits, rows, error)
    return continiousFrame.to_bytes() + endFrame.to_bytes()

def make_select_object(sql, resp_content, req_params = None, output_raw = False, simulate_bad_frame = False, simulate_bad_crc = False, status = 206, error = b''):
    req_body = xml_utils.to_select_object(sql, req_params)
    process_type = 'csv'
    if (req_params is not None and 'Json_Type' in req_params):
        process_type = 'json'

    request_text = '''POST /select-test.txt?x-oss-process={2}/select HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: {0}
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:W6whAowN4aImQ0dfbMHyFfD0t1g=
Accept: */*

{1}'''.format(len(req_body), to_string(req_body), process_type)
    resp_body = generate_data(resp_content, output_raw, simulate_bad_crc=simulate_bad_crc, status=status, error = error)
    if (simulate_bad_frame):
        resp_body = resp_content

    response_text = '''HTTP/1.1 206 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"
x-oss-select-output-raw:{1}

'''.format(len(resp_body), str(output_raw).lower())

    if (sys.version_info[0] == 3 and type(resp_body).__name__ == 'str'):
        resp_body = str.encode(resp_body)

    return request_text, str.encode(response_text) + resp_body

def make_head_object(req_params, scanned_size, splits, rows, cols, status, error):
    req_body = xml_utils.to_get_select_object_meta(req_params)
    process_type = 'csv'
    if (req_params is not None and 'Json_Type' in req_params):
        process_type = 'json'

    request_text = '''POST /select-test.txt?x-oss-process={2}/meta HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: {0}
date: Sat, 12 Dec 2015 00:35:53 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
authorization: OSS ZCDmm7TPZKHtx77j:W6whAowN4aImQ0dfbMHyFfD0t1g=
Accept: */*


{1}'''.format(len(req_body), to_string(req_body), process_type)

    resp_body = generate_head_data(scanned_size, splits, rows, cols, error, status)
    response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:53 GMT
Content-Length: {0}
Connection: keep-alive
x-oss-request-id: 566B6BE93A7B8CFD53D4BAA3
ETag: "D80CF0E5BE2436514894D64B2BCFB2AE"

'''.format(len(resp_body))

    return request_text, str.encode(response_text) + resp_body

def callback(offset, length):
    print(offset)
    print(length)

class SelectCaseHelper(object):
    def create_meta(self, tester, do_request, head_params = None, error = b'', status = 200):
        scanned_size = 10000
        splits = 100
        rows = 1000
        cols = 20

        if (head_params is not None and 'Json_Type' in head_params):
            req, resp = make_head_object(head_params, scanned_size, splits, rows, None, status, error)
        else:
            req, resp = make_head_object(head_params, scanned_size, splits, rows, cols, status, error)

        req_info = mock_response(do_request, resp)

        result = bucket().create_select_object_meta('select-test.txt', head_params)

        tester.assertRequest(req_info, req)
        tester.assertEqual(result.rows, rows)
        tester.assertEqual(result.splits, splits)
        tester.assertEqual(result.request_id, '566B6BE93A7B8CFD53D4BAA3')
    
    def select(self, tester, do_request, callback = None, select_params = None, byte_range = None):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
        if select_params is not None and 'Json_Type' in select_params:
            if select_params['Json_Type'] == 'DOCUMENT':
                resp_content = b'{contacts:[{\"firstName\":\"John\", \"lastName\":\"Smith\"}]}'
            else:
                resp_content = b'{\"firstName\":\"John\", \"lastName\":\"Smith\"}' 

        output_raw = False

        if (select_params is not None and 'OutputRawData' in select_params and select_params['OutputRawData']):
            output_raw = True
        req, resp = make_select_object(sql, resp_content, select_params, output_raw)

        req_info = mock_response(do_request, resp)

        result = bucket().select_object('select-test.txt', sql, callback, select_params, byte_range)

        tester.assertEqual(result.status, 206)
        tester.assertRequest(req_info, req)
        
        if (result.status//100 == 2):
            content = result.read()
            tester.assertEqual(content, resp_content)

class TestSelectObject(OssTestCase):
    
    @patch('oss2.Session.do_request')
    def test_create_csv_meta_with_none_params(self, do_request):
        head_params = None
        helper = SelectCaseHelper()
        helper.create_meta(self, do_request, head_params)
    
    @patch('oss2.Session.do_request')
    def test_create_csv_meta_with_params(self, do_request):
        head_params = {'RecordDelimiter':'\n', 'FieldDelimiter':',', 'QuoteCharacter':'"', 'CompressionType':'None', 'OverwriteIfExists':'True'}
        helper = SelectCaseHelper()
        helper.create_meta(self, do_request, head_params)

    @patch('oss2.Session.do_request')
    def test_create_csv_meta_with_invalid_params(self, do_request):
        head_params = {'InvalidParam':'\n'}
        helper = SelectCaseHelper()
        self.assertRaises(SelectOperationClientError, helper.create_meta, self, do_request, head_params)

    @patch('oss2.Session.do_request')
    def test_select_csv(self, do_request):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
        req, resp = make_select_object(sql, resp_content)

        req_info = mock_response(do_request, resp)

        result = bucket().select_object('select-test.txt', sql, None)

        self.assertEqual(result.status, 206)
        self.assertRequest(req_info, req)
        
        content = b''
        for chunk in result:
            content += chunk
        
        self.assertEqual(content, resp_content)
    
    @patch('oss2.Session.do_request')
    def test_select_csv_next(self, do_request):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
        req, resp = make_select_object(sql, resp_content)

        req_info = mock_response(do_request, resp)

        result = bucket().select_object('select-test.txt', sql, None)

        self.assertEqual(result.status, 206)
        self.assertRequest(req_info, req)
        
        content = b''
        result_iter = iter(result)
        content = next(result_iter)
        
        self.assertEqual(content, resp_content)

    @patch('oss2.Session.do_request')
    def test_select_csv_read(self, do_request):
        helper = SelectCaseHelper()
        helper.select(self, do_request)
    
    @patch('oss2.Session.do_request')
    def test_select_csv_read_with_params(self, do_request):
        select_params = {'CsvHeaderInfo':'Use', 'CommentCharacter':'#', 'RecordDelimiter':'\n', 'OutputRecordDelimiter':'\n',
                         'FieldDelimiter':',', 'OutputFieldDelimiter':',', 'QuoteCharacter':'"', 'SplitRange':[0,10], 'CompressionType':'GZIP',
                         'KeepAllColumns':True, 'OutputRawData':False, 'EnablePayloadCrc':True, 'OutputHeader':False, 'SkipPartialDataRecord':False}
        helper = SelectCaseHelper()
        helper.select(self, do_request, None, select_params)
    
    @patch('oss2.Session.do_request')
    def test_select_csv_read_output_raw(self, do_request):
        select_params = {'OutputRawData':True}
        helper = SelectCaseHelper()
        helper.select(self, do_request, None, select_params)

    @patch('oss2.Session.do_request')
    def test_select_csv_with_bad_response(self, do_request):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
       
        req, resp = make_select_object(sql, resp_content, simulate_bad_frame=True)

        req_info = mock_response(do_request, resp)

        try:
            result = bucket().select_object('select-test.txt', sql)
            result.read()
            self.assertFalse(True, "expect SelectOperationClientError")
        except SelectOperationClientError:
            pass

    @patch('oss2.Session.do_request')
    def test_select_csv_with_bad_crc(self, do_request):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
        select_params = {'EnablePayloadCrc':True}

        req, resp = make_select_object(sql, resp_content, simulate_bad_crc=True)

        req_info = mock_response(do_request, resp)

        try:
            result = bucket().select_object('select-test.txt', sql, None, select_params)
            result.read()
            self.assertFalse(True, "expect InconsistentError")
        except InconsistentError:
            pass

    @patch('oss2.Session.do_request')
    def test_select_csv_with_callback(self, do_request):
        helper = SelectCaseHelper()
        helper.select(self, do_request, callback)
    
    @patch('oss2.Session.do_request')
    def test_select_csv_with_error(self, do_request):
        sql = "select * from ossobject limit 10"
        resp_content = b'a,b,c,d,e,f,,n,g,l,o,p'
        select_params = {'EnablePayloadCrc':True}

        req, resp = make_select_object(sql, resp_content, status=400, error = b"test error")

        req_info = mock_response(do_request, resp)

        try:
            result = bucket().select_object('select-test.txt', sql, None, select_params)
            result.read()
            self.assertFalse(True, "expect SelectOperationFailed")
        except SelectOperationFailed as errorException:
            self.assertEqual(errorException.status, 400)
            self.assertEqual(errorException.message, b'test error')
    
    @patch('oss2.Session.do_request')
    def test_head_csv_with_error(self, do_request):
        head_params = None
        helper = SelectCaseHelper()
        try:
            helper.create_meta(self, do_request, head_params, b'InvalidCsvLine.error code:invalid csv', 400)
            self.assertFalse(True, "expect SelectOperationFailed")
        except SelectOperationFailed as errorException:
            self.assertEqual(errorException.status, 400)
            self.assertEqual(errorException.message, b'error code:invalid csv')
            self.assertEqual(errorException.code, b'InvalidCsvLine')
    
    @patch('oss2.Session.do_request')
    def test_select_json_read_with_params(self, do_request):
        select_params = {'CompressionType':'GZIP',
                         'OutputRawData':False, 'EnablePayloadCrc':True, 'OutputRecordDelimiter':',\n', 'SkipPartialDataRecord':False, 'MaxSkippedRecordsAllowed':100, 'Json_Type':'DOCUMENT'}
        helper = SelectCaseHelper()
        helper.select(self, do_request, None, select_params)

    @patch('oss2.Session.do_request')
    def test_select_json_read_with_line_range(self, do_request):
        select_params = {'LineRange':[0,10], 'CompressionType':'None',
                         'OutputRawData':True, 'EnablePayloadCrc':False, 'OutputRecordDelimiter':',\n', 'SkipPartialDataRecord':True, 'MaxSkippedRecordsAllowed':100, 'Json_Type':'LINES'}
        helper = SelectCaseHelper()
        helper.select(self, do_request, None, select_params)

    @patch('oss2.Session.do_request')
    def test_select_json_read_with_split_range(self, do_request):
        select_params = {'SplitRange':[0,10], 'CompressionType':'None',
                         'OutputRawData':True, 'EnablePayloadCrc':False, 'OutputRecordDelimiter':',\n', 'SkipPartialDataRecord':True, 'MaxSkippedRecordsAllowed':100, 'Json_Type':'LINES',
                         'ParseJsonNumberAsString':False}
        helper = SelectCaseHelper()
        helper.select(self, do_request, None, select_params)

    @patch('oss2.Session.do_request')
    def test_select_json_read_with_invalid_parameters(self, do_request):
        select_params = {'SplitRangeXXXX':[0,10]}
        helper = SelectCaseHelper()
        try:
            helper.select(self, do_request, None, select_params)
            helper.assertFalse()
        except SelectOperationClientError:
            print("expected error")

    @patch('oss2.Session.do_request')
    def test_select_json_with_invalid_parameters(self, do_request):
        head_params = {'Invalid':'\n', 'Json_Type':'LINES'}
        helper = SelectCaseHelper()
        self.assertRaises(SelectOperationClientError, helper.create_meta, self, do_request, head_params)

    @patch('oss2.Session.do_request')
    def test_create_json_meta_with_params(self, do_request):
        head_params = {'OverwriteIfExists':'True', 'Json_Type':'LINES','CompressionType':'NONE'}
        helper = SelectCaseHelper()
        helper.create_meta(self, do_request, head_params)

    @patch('oss2.Session.do_request')
    def test_create_json_meta_with_params2(self, do_request):
        head_params = {'OverwriteIfExists':'True', 'Json_Type':'DOCUMENT'}
        helper = SelectCaseHelper()
        try:
            helper.create_meta(self, do_request, head_params)
            helper.assertFalse()
        except SelectOperationClientError:
            print("expected error")

    @patch('oss2.Session.do_request')
    def test_range_select(self, do_request):
        byte_range = [0,9]
        select_params = {}
        helper = SelectCaseHelper()
        try:
            helper.select(self, do_request, None, select_params, byte_range)
            helper.assertFalse()
        except ClientError:
            print("expected error")
        
        select_params = {'AllowQuotedRecordDelimiter':False}
        helper.select(self, do_request, None, select_params, byte_range)

        select_params['AllowQuotedRecordDelimiter'] = True 
        try:
            helper.select(self, do_request, None, select_params, byte_range)
            helper.assertFalse()
        except ClientError:
            print("expected error")
        
        try:
            helper.select(self, do_request, None, None, byte_range)
            helper.assertFalse()
        except ClientError:
            print("expected error")

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import os
import random
import string
import unittest
import time
import tempfile
import errno
import logging
from Crypto.PublicKey import RSA
from Crypto.PublicKey.RSA import RsaKey
import oss2

logging.basicConfig(level=logging.DEBUG)

OSS_ID = os.getenv("OSS_TEST_ACCESS_KEY_ID")
OSS_SECRET = os.getenv("OSS_TEST_ACCESS_KEY_SECRET")
OSS_ENDPOINT = os.getenv("OSS_TEST_ENDPOINT")
OSS_TEST_BUCKET = os.getenv("OSS_TEST_BUCKET")
OSS_CNAME = os.getenv("OSS_TEST_CNAME")
OSS_REGION = os.getenv("OSS_TEST_REGION", "cn-hangzhou")

OSS_CMK = os.getenv("OSS_TEST_KMS_CMK_ID")
OSS_CMK_REGION = os.getenv("OSS_TEST_KMS_REGION")

OSS_STS_ID = os.getenv("OSS_TEST_STS_ID")
OSS_STS_KEY = os.getenv("OSS_TEST_STS_KEY")
OSS_STS_ARN = os.getenv("OSS_TEST_STS_ARN")

OSS_PAYER_UID = os.getenv("OSS_TEST_PAYER_UID")
OSS_PAYER_ID = os.getenv("OSS_TEST_PAYER_ACCESS_KEY_ID")
OSS_PAYER_SECRET = os.getenv("OSS_TEST_PAYER_ACCESS_KEY_SECRET")

OSS_INVENTORY_BUCKET_DESTINATION_ARN = os.getenv("OSS_TEST_RAM_ROLE_ARN")
OSS_INVENTORY_BUCKET_DESTINATION_ACCOUNT = os.getenv("OSS_TEST_RAM_UID")

OSS_AUTH_VERSION = None
OSS_TEST_AUTH_SERVER_HOST = os.getenv("OSS_TEST_AUTH_SERVER_HOST")

private_key = RSA.generate(1024)
public_key = private_key.publickey()
private_key_str = RsaKey.exportKey(private_key)
public_key_str = RsaKey.exportKey(public_key)
key_pair = {'private_key': private_key_str, 'public_key': public_key_str}

private_key_compact = '''-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCokfiAVXXf5ImFzKDw+XO/UByW6mse2QsIgz3ZwBtMNu59fR5z
ttSx+8fB7vR4CN3bTztrP9A6bjoN0FFnhlQ3vNJC5MFO1PByrE/MNd5AAfSVba93
I6sx8NSk5MzUCA4NJzAUqYOEWGtGBcom6kEF6MmR1EKib1Id8hpooY5xaQIDAQAB
AoGAOPUZgkNeEMinrw31U3b2JS5sepG6oDG2CKpPu8OtdZMaAkzEfVTJiVoJpP2Y
nPZiADhFW3e0ZAnak9BPsSsySRaSNmR465cG9tbqpXFKh9Rp/sCPo4Jq2n65yood
JBrnGr6/xhYvNa14sQ6xjjfSgRNBSXD1XXNF4kALwgZyCAECQQDV7t4bTx9FbEs5
36nAxPsPM6aACXaOkv6d9LXI7A0J8Zf42FeBV6RK0q7QG5iNNd1WJHSXIITUizVF
6aX5NnvFAkEAybeXNOwUvYtkgxF4s28s6gn11c5HZw4/a8vZm2tXXK/QfTQrJVXp
VwxmSr0FAajWAlcYN/fGkX1pWA041CKFVQJAG08ozzekeEpAuByTIOaEXgZr5MBQ
gBbHpgZNBl8Lsw9CJSQI15wGfv6yDiLXsH8FyC9TKs+d5Tv4Cvquk0efOQJAd9OC
lCKFs48hdyaiz9yEDsc57PdrvRFepVdj/gpGzD14mVerJbOiOF6aSV19ot27u4on
Td/3aifYs0CveHzFPQJAWb4LCDwqLctfzziG7/S7Z74gyq5qZF4FUElOAZkz718E
yZvADwuz/4aK0od0lX9c4Jp7Mo5vQ4TvdoBnPuGoyw==
-----END RSA PRIVATE KEY-----'''

public_key_compact = '''-----BEGIN RSA PUBLIC KEY-----
MIGJAoGBAKiR+IBVdd/kiYXMoPD5c79QHJbqax7ZCwiDPdnAG0w27n19HnO21LH7
x8Hu9HgI3dtPO2s/0DpuOg3QUWeGVDe80kLkwU7U8HKsT8w13kAB9JVtr3cjqzHw
1KTkzNQIDg0nMBSpg4RYa0YFyibqQQXoyZHUQqJvUh3yGmihjnFpAgMBAAE=
-----END RSA PUBLIC KEY-----'''

key_pair_compact = {'private_key': private_key_compact, 'public_key': public_key_compact}


def random_string(n):
    return ''.join(random.choice(string.ascii_lowercase) for i in range(n))

OSS_BUCKET = ''
if OSS_TEST_BUCKET is None:
    OSS_BUCKET = 'oss-python-sdk-'+random_string(10)
else:
    OSS_BUCKET = OSS_TEST_BUCKET + random_string(10)

def random_bytes(n):
    return oss2.to_bytes(random_string(n))

def clean_and_delete_bucket(bucket):
    # check if bucket is in versioning status
    try:
        result = bucket.get_bucket_info()
        if result.versioning_status in [oss2.BUCKET_VERSIONING_ENABLE, oss2.BUCKET_VERSIONING_SUSPEND]:
            next_key_marker = None
            next_versionid_marker = None
            is_truncated = True
            while is_truncated is True:
                objects = bucket.list_object_versions(key_marker=next_key_marker, versionid_marker=next_versionid_marker)
                for obj in objects.versions:
                    bucket.delete_object(obj.key, params={'versionId': obj.versionid})
                for del_marker in objects.delete_marker:
                    bucket.delete_object(del_marker.key, params={'versionId': del_marker.versionid})
                is_truncated = objects.is_truncated
                if is_truncated:
                    next_key_marker = objects.next_key_marker
                    next_versionid_marker = objects.next_versionid_marker
    except:
        pass

    # list all upload_parts to delete
    up_iter = oss2.MultipartUploadIterator(bucket)
    for up in up_iter:
        bucket.abort_multipart_upload(up.key, up.upload_id)

    # list all objects to delete
    obj_iter = oss2.ObjectIterator(bucket)
    for obj in obj_iter:
        bucket.delete_object(obj.key)
    
    # list all live channels to delete
    for ch_iter in oss2.LiveChannelIterator(bucket):
        bucket.delete_live_channel(ch_iter.name)

    # delete_bucket
    bucket.delete_bucket()

def clean_and_delete_bucket_by_prefix(bucket_prefix):
    service = oss2.Service(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT)
    buckets = service.list_buckets(prefix=bucket_prefix).buckets
    for b in buckets:
        bucket = oss2.Bucket(oss2.Auth(OSS_ID, OSS_SECRET), b.extranet_endpoint, b.name)
        clean_and_delete_bucket(bucket)
        

def delete_keys(bucket, key_list):
    if not key_list:
        return

    n = 100
    grouped = [key_list[i:i+n] for i in range(0, len(key_list), n)]
    for g in grouped:
        bucket.batch_delete_objects(g)


class NonlocalObject(object):
    def __init__(self, value):
        self.var = value


def wait_meta_sync():
    if os.environ.get('TRAVIS'):
        time.sleep(5)
    else:
        time.sleep(1)


class OssTestCase(unittest.TestCase):
    SINGLE_THREAD_CASE = 'single thread case'

    def __init__(self, *args, **kwargs):
        super(OssTestCase, self).__init__(*args, **kwargs)
        self.bucket = None
        self.prefix = random_string(12)
        self.default_connect_timeout = oss2.defaults.connect_timeout
        self.default_multipart_num_threads = oss2.defaults.multipart_threshold

        self.default_multiget_threshold = 1024 * 1024
        self.default_multiget_part_size = 100 * 1024

    def setUp(self):
        oss2.defaults.connect_timeout = self.default_connect_timeout
        oss2.defaults.multipart_threshold = self.default_multipart_num_threads
        oss2.defaults.multipart_num_threads = random.randint(1, 5)

        oss2.defaults.multiget_threshold = self.default_multiget_threshold
        oss2.defaults.multiget_part_size = self.default_multiget_part_size
        oss2.defaults.multiget_num_threads = random.randint(1, 5)

        global OSS_AUTH_VERSION
        OSS_AUTH_VERSION = os.getenv('OSS_TEST_AUTH_VERSION')

        self.bucket = oss2.Bucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT, OSS_BUCKET)

        try:
            self.bucket.create_bucket()
        except:
            pass

        self.rsa_crypto_bucket = oss2.CryptoBucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT,
                                                   OSS_BUCKET, crypto_provider=oss2.RsaProvider(key_pair))

        self.kms_crypto_bucket = oss2.CryptoBucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT,
                                                   OSS_BUCKET, crypto_provider=oss2.AliKMSProvider(OSS_ID, OSS_SECRET,
                                                                                                   OSS_REGION, OSS_CMK))

        self.key_list = []
        self.temp_files = []

    def tearDown(self):
        for temp_file in self.temp_files:
            oss2.utils.silently_remove(temp_file)

        clean_and_delete_bucket(self.bucket)
        clean_and_delete_bucket_by_prefix(OSS_BUCKET + "-test-")

    def random_key(self, suffix=''):
        key = self.prefix + random_string(12) + suffix
        self.key_list.append(key)

        return key

    def random_filename(self):
        filename = random_string(16)
        self.temp_files.append(filename)

        return filename

    def _prepare_temp_file(self, content):
        fd, pathname = tempfile.mkstemp(suffix='test-upload')

        os.write(fd, content)
        os.close(fd)

        self.temp_files.append(pathname)
        return pathname

    def _prepare_temp_file_with_size(self, size):
        fd, pathname = tempfile.mkstemp(suffix='test-upload')

        block_size = 8 * 1024 * 1024
        num_written = 0

        while num_written < size:
            to_write = min(block_size, size - num_written)
            num_written += to_write

            content = 's' * to_write
            os.write(fd, oss2.to_bytes(content))

        os.close(fd)

        self.temp_files.append(pathname)
        return pathname

    def retry_assert(self, func):
        for i in range(5):
            if func():
                return
            else:
                time.sleep(i+2)

        self.assertTrue(False)

    def assertFileContent(self, filename, content):
        with open(filename, 'rb') as f:
            read = f.read()
            self.assertEqual(len(read), len(content))
            self.assertEqual(read, content)

    def assertFileContentNotEqual(self, filename, content):
        with open(filename, 'rb') as f:
            read = f.read()
            self.assertNotEqual(len(read), len(content))
            self.assertNotEqual(read, content)




# -*- coding: utf-8 -*-
import json
import unittest
import os

from aliyunsdkcore import client

import oss2
import unittests
from oss2 import LocalRsaProvider, AliKMSProvider, RsaProvider, compat
from oss2.utils import AESCipher, silently_remove
from oss2.exceptions import OpenApiServerError, OpenApiFormatError, ClientError
from mock import patch

from .common import OSS_ID, OSS_SECRET, OSS_REGION, OSS_CMK_REGION, OSS_CMK, OSS_STS_ID, OSS_STS_ARN, OSS_STS_KEY, random_string, \
    key_pair, key_pair_compact
from aliyunsdksts.request.v20150401 import AssumeRoleRequest
from Crypto.PublicKey import RSA
from Crypto.PublicKey.RSA import RsaKey
import random


class TestCrypto(unittests.common.OssTestCase):
    # 测试初始化LocalRsaProvider时未初始化cipher，此时应该抛出异常
    def test_rsa_provider_init_cipher_is_none(self):
        self.assertRaises(ClientError, LocalRsaProvider, dir='./', key='rsa-test', cipher=None)
        self.assertRaises(ClientError, RsaProvider, key_pair=key_pair, cipher=None)

    # 测试当keys不存在时, local_rsa_provider会创建一个默认的密钥对
    def test_rsa_provider_init_keys_not_exist(self):
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')
        provider = LocalRsaProvider(dir='./', key='rsa-test')
        self.assertTrue(os.path.exists('./rsa-test.public_key.pem'))
        self.assertTrue(os.path.exists('./rsa-test.private_key.pem'))
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

    # 测试keys内容不是对称rsa密钥时，初始化LocalRsaProvider时抛出异常
    def test_rsa_provider_init_invalid_keys(self):
        private_key = RSA.generate(2048)
        public_key = private_key.publickey()
        # 这个地方修改private_key的内容
        private_key = random_string(2048)

        with open('./rsa-test.private_key.pem', 'wb') as f:
            f.write(oss2.to_bytes(private_key))

        with open('./rsa-test.public_key.pem', 'wb') as f:
            f.write(public_key.exportKey())

        self.assertRaises(ClientError, LocalRsaProvider, dir='./', key='rsa-test')
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        self.assertRaises(ClientError, RsaProvider, key_pair={'private_key': private_key, 'public_key': public_key})

    # 测试当keys存在时，使用错误的passpass初始化LocalRsaProvider时抛出异常
    def test_rsa_provider_init_invalid_passphrase(self):
        private_key = RSA.generate(2048)
        public_key = private_key.publickey()
        passphrase = random_string(6)
        invalid_passphrase = random_string(8)

        with open('./rsa-test.private_key.pem', 'wb') as f:
            f.write(private_key.exportKey(passphrase=passphrase))

        with open('./rsa-test.public_key.pem', 'wb') as f:
            f.write(public_key.exportKey(passphrase=passphrase))

        self.assertRaises(ClientError, LocalRsaProvider, dir='./', key='rsa-test', passphrase=invalid_passphrase)
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        private_key_str = RsaKey.exportKey(private_key, passphrase=passphrase)
        public_key_str = RsaKey.exportKey(public_key, passphrase=passphrase)

        self.assertRaises(ClientError, RsaProvider,
                          key_pair={'private_key': private_key_str, 'public_key': public_key_str},
                          passphrase=invalid_passphrase)

    # 测试基本key, start加/解密
    def test_rsa_provider_basic(self):
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        provider = LocalRsaProvider(dir='./', key='rsa-test', passphrase=random_string(8))
        self.assertEqual(provider.wrap_alg, "RSA/NONE/OAEPWithSHA-1AndMGF1Padding")
        self.assertEqual(provider.cipher.alg, "AES/CTR/NoPadding")
        plain_key = provider.get_key()
        self.assertEqual(len(plain_key), provider.cipher.key_len)
        plain_iv = provider.get_iv()

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()
                self.assertFalse(content_crypto_material.is_unencrypted())
                decrypted_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
                decrypted_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
                self.assertEqual(plain_key, decrypted_key)
                self.assertEqual(plain_iv, decrypted_iv)

        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        provider = RsaProvider(key_pair=key_pair, passphrase=random_string(8))
        self.assertEqual(provider.wrap_alg, "RSA/NONE/PKCS1Padding")
        self.assertEqual(provider.cipher.alg, "AES/CTR/NoPadding")
        plain_key = provider.get_key()
        self.assertEqual(len(plain_key), provider.cipher.key_len)
        plain_iv = provider.get_iv()

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()
                self.assertFalse(content_crypto_material.is_unencrypted())
                decrypted_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
                decrypted_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
                self.assertEqual(plain_key, decrypted_key)
                self.assertEqual(plain_iv, decrypted_iv)

    # 测试使用不同的rsa keys的provider
    def test_local_rsa_provider_diff_keys(self):
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')
        silently_remove('./rsa-test-diff.public_key.pem')
        silently_remove('./rsa-test-diff.private_key.pem')

        provider = LocalRsaProvider(dir='./', key='rsa-test')
        provider_diff = LocalRsaProvider(dir='./', key='rsa-test-diff')

        plain_key = provider.get_key()
        plain_iv = provider.get_iv()

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()
                self.assertFalse(content_crypto_material.is_unencrypted())
                self.assertRaises(ClientError, provider_diff.decrypt_encrypted_key,
                                  content_crypto_material.encrypted_key)
                self.assertRaises(ClientError, provider_diff.decrypt_encrypted_iv,
                                  content_crypto_material.encrypted_iv)

        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')
        silently_remove('./rsa-test-diff.public_key.pem')
        silently_remove('./rsa-test-diff.private_key.pem')

        provider = RsaProvider(key_pair=key_pair)
        provider_diff = RsaProvider(key_pair=key_pair_compact)

        plain_key = provider.get_key()
        plain_iv = provider.get_iv()

        with patch.object(oss2.utils, 'random_key', return_value=plain_key, autospect=True):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()
                self.assertFalse(content_crypto_material.is_unencrypted())
                self.assertRaises(ClientError, provider_diff.decrypt_encrypted_key,
                                  content_crypto_material.encrypted_key)
                self.assertRaises(ClientError, provider_diff.decrypt_encrypted_iv,
                                  content_crypto_material.encrypted_iv)

    def test_rsa_provider_adapter(self):
        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        provider = LocalRsaProvider(dir='./', key='rsa-test')
        content = b'a' * random.randint(1, 100) * 1024
        content_crypto_material = provider.create_content_material()
        plain_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
        plain_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
        cipher = content_crypto_material.cipher

        stream_encrypted = provider.make_encrypt_adapter(content, cipher)
        encrypted_content = stream_encrypted.read()
        # reset cipher
        cipher.initialize(plain_key, plain_iv)
        stream_decrypted = provider.make_decrypt_adapter(encrypted_content, cipher)
        self.assertEqual(content, stream_decrypted.read())

        silently_remove('./rsa-test.public_key.pem')
        silently_remove('./rsa-test.private_key.pem')

        provider = RsaProvider(key_pair)
        content = b'b' * random.randint(1, 100) * 1024
        content_crypto_material = provider.create_content_material()
        plain_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
        plain_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
        cipher = content_crypto_material.cipher

        stream_encrypted = provider.make_encrypt_adapter(content, cipher)
        encrypted_content = stream_encrypted.read()
        # reset cipher
        cipher.initialize(plain_key, plain_iv)
        stream_decrypted = provider.make_decrypt_adapter(encrypted_content, cipher)
        self.assertEqual(content, stream_decrypted.read())

    # 测试初始化AliKMSProvider时未初始化cipher，此时应该抛出异常
    def test_ali_kms_provider_init_cipher_is_none(self):
        id, key, token = self.get_sts()
        self.assertRaises(ClientError, AliKMSProvider, access_key_id=id, access_key_secret=key, region=OSS_REGION,
                          cmk_id=OSS_CMK, cipher=None)

    # 测试基本key, start加/解密
    def test_ali_kms_provider_basic(self):
        provider = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK, passphrase=random_string(8))
        self.assertEqual(provider.wrap_alg, "KMS/ALICLOUD")
        self.assertEqual(provider.cipher.alg, "AES/CTR/NoPadding")
        plain_key, encrypted_key = provider.get_key()
        plain_iv = provider.get_iv()

        with patch('oss2.AliKMSProvider.get_key', return_value=[plain_key, encrypted_key]):
            with patch.object(oss2.utils, 'random_iv', return_value=plain_iv, autospect=True):
                content_crypto_material = provider.create_content_material()
                self.assertFalse(content_crypto_material.is_unencrypted())
                decrypted_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
                decrypted_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
                self.assertEqual(plain_key, decrypted_key)
                self.assertEqual(plain_iv, decrypted_iv)

    # 测试使用不同的passphrase解析加密key和start抛出异常
    def test_ali_kms_provider_diff_passphrase(self):
        provider = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK, passphrase=random_string(6))
        plain_key, encrypted_key = provider.get_key()
        encrypted_iv = provider.get_iv()

        provider_diff = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK, passphrase=random_string(8))
        self.assertRaises(OpenApiServerError, provider_diff.decrypt_encrypted_key, encrypted_key)
        self.assertRaises(OpenApiServerError, provider_diff.decrypt_encrypted_iv, encrypted_iv)

    # 测试使用不同的region解析加密key和start时抛出异常
    def test_ali_kms_provider_invalid_region(self):
        provider = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK)
        plain_key, encrypted_key = provider.get_key()
        encrypted_iv = provider.get_iv()

        region_list = ['cn-hangzhou', 'cn-shanghai', 'cn-qingdao', 'cn-beijing', 'cn-zhangjiakou',
                       'cn-huhehaote', 'cn-shenzhen', 'cn-hongkong', 'us-west-1', 'us-east-1',
                       'ap-southeast-1', 'ap-southeast-2', 'ap-southeast-3', 'ap-southeast-5',
                       'ap-northeast-1', 'ap-south-1', 'eu-central-1', 'eu-west-1', 'me-east-1']

        if OSS_REGION in region_list:
            region_list.remove(OSS_REGION)

        region_num = len(region_list)
        invalid_region = region_list[random.randint(0, region_num - 1)]

        provider_invalid = AliKMSProvider(OSS_ID, OSS_SECRET, invalid_region, OSS_CMK)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_key)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_iv)

    # 测试使用不同的ak解析加密key和start的值时抛出异常
    def test_ali_kms_provider_invalid_ak(self):
        provider = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK)
        plain_key, encrypted_key = provider.get_key()
        encrypted_iv = provider.get_iv()

        invalid_secret = random_string(len(OSS_SECRET))
        provider_invalid = AliKMSProvider(OSS_ID, invalid_secret, OSS_CMK_REGION, OSS_CMK)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_key)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_iv)

        invald_id = random_string(len(OSS_ID))
        provider_invalid = AliKMSProvider(invald_id, OSS_SECRET, OSS_CMK_REGION, OSS_CMK)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_key)
        self.assertRaises(OpenApiServerError, provider_invalid.decrypt_encrypted_key, encrypted_iv)

    # 测试kms服务返回错误的情况
    def test_ali_kms_with_error_response(self):
        if oss2.compat.is_py33:
            return

        kms = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK)

        # 模拟返回的数据格式不对，不是正确的json格式字符串
        plain_key = random_string(32)
        encrypted_key = random_string(32)
        return_value = "{'Plaintext': %s, 'CiphertextBlob': %s}" % (plain_key, encrypted_key)
        with patch.object(client.AcsClient, 'do_action_with_exception', return_value=return_value, autospect=True):
            self.assertRaises(OpenApiFormatError, kms.get_key)

    def test_ali_kms_provider_adapter(self):
        provider = AliKMSProvider(OSS_ID, OSS_SECRET, OSS_CMK_REGION, OSS_CMK)
        content = b'a' * random.randint(1, 100) * 1024
        content_crypto_material = provider.create_content_material()

        plain_key = provider.decrypt_encrypted_key(content_crypto_material.encrypted_key)
        plain_iv = provider.decrypt_encrypted_iv(content_crypto_material.encrypted_iv)
        cipher = content_crypto_material.cipher

        stream_encrypted = provider.make_encrypt_adapter(content, cipher)
        encrypted_content = stream_encrypted.read()
        # reset cipher
        cipher.initialize(plain_key, plain_iv)
        stream_decrypted = provider.make_decrypt_adapter(encrypted_content, cipher)
        self.assertEqual(content, stream_decrypted.read())

        # 使用不同的content crypto material
        content_crypto_material_diff = provider.create_content_material()
        plain_key = provider.decrypt_encrypted_key(content_crypto_material_diff.encrypted_key)
        plain_iv = provider.decrypt_encrypted_iv(content_crypto_material_diff.encrypted_iv)
        cipher = content_crypto_material_diff.cipher

        stream_encrypted_diff = provider.make_encrypt_adapter(content, cipher)
        encrypted_content_diff = stream_encrypted_diff.read()
        self.assertNotEqual(encrypted_content_diff, encrypted_content)
        # reset cipher
        cipher.initialize(plain_key, plain_iv)
        stream_decrypted_diff = provider.make_decrypt_adapter(encrypted_content_diff, cipher)
        self.assertEqual(content, stream_decrypted_diff.read())

    def get_sts(self):
        clt = client.AcsClient(OSS_STS_ID, OSS_STS_KEY, OSS_REGION)
        req = AssumeRoleRequest.AssumeRoleRequest()

        req.set_accept_format('json')
        req.set_RoleArn(OSS_STS_ARN)
        req.set_RoleSessionName('oss-python-sdk-example')

        body = clt.do_action_with_exception(req)

        j = json.loads(compat.to_unicode(body))

        return j['Credentials']['AccessKeyId'], j['Credentials']['AccessKeySecret'], j['Credentials'][
            'SecurityToken']



# -*- coding: utf-8 -*-

import os
import oss2
from oss2.models import BucketQosInfo

# 以下代码展示User以及Bucket的QoSInfo的操作示例

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你要请求的Bucket名称>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')

# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param

# 以下代码展示user qos info的get操作示例
# 获取user qos info
service = oss2.Service(oss2.Auth(access_key_id, access_key_secret), endpoint)
user_qos_info = service.get_user_qos_info()
print('===Get user qos info===')
print('region:', user_qos_info.region)
print('total_upload_bw:', user_qos_info.total_upload_bw)
print('intranet_upload_bw:', user_qos_info.intranet_upload_bw)
print('extranet_upload_bw:', user_qos_info.extranet_upload_bw)
print('total_download_bw:', user_qos_info.total_download_bw)
print('intranet_download_bw:', user_qos_info.intranet_download_bw)
print('extranet_download_bw:', user_qos_info.extranet_download_bw)
print('total_qps:', user_qos_info.total_qps)
print('intranet_qps:', user_qos_info.intranet_qps)
print('extranet_qps:', user_qos_info.extranet_qps)

# 以下代码展示bucket qos info的put get delete操作示例
# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

# 创建BucketQosInfo对象, -1表示不做单独限制, 带宽单位为Gbps，具体设置规则请参考官网文档
bucket_qos_info = BucketQosInfo(
                    total_upload_bw = -1,
                    intranet_upload_bw = 2,
                    extranet_upload_bw = 2,
                    total_download_bw = -1,
                    intranet_download_bw = -1,
                    extranet_download_bw = -1,
                    total_qps = -1,
                    intranet_qps = -1,
                    extranet_qps = -1)

# 设置bucket qos info
result = bucket.put_bucket_qos_info(bucket_qos_info)
print('http response status:', result.status)

# 获取bucket qos info
bucket_qos_info = bucket.get_bucket_qos_info()
print('===Get bucket qos info===')
print('total_upload_bw:', bucket_qos_info.total_upload_bw)
print('intranet_upload_bw:', bucket_qos_info.intranet_upload_bw)
print('extranet_upload_bw:', bucket_qos_info.extranet_upload_bw)
print('total_download_bw:', bucket_qos_info.total_download_bw)
print('intranet_download_bw:', bucket_qos_info.intranet_download_bw)
print('extranet_download_bw:', bucket_qos_info.extranet_download_bw)
print('total_qps:', bucket_qos_info.total_qps)
print('intranet_qps:', bucket_qos_info.intranet_qps)
print('extranet_qps:', bucket_qos_info.extranet_qps)

# 删除bucket qos info配置
result = bucket.delete_bucket_qos_info()
print('http response status:', result.status)


# -*- coding: utf-8 -*-

import requests

from .common import *

if oss2.compat.is_py2:
    from aliyunsdkcore import client
    from aliyunsdksts.request.v20150401 import AssumeRoleRequest

    import json


    class StsToken(object):
        def __init__(self):
            self.access_key_id = ''
            self.access_key_secret = ''
            self.expiration = 0
            self.security_token = ''
            self.request_id = ''


    def fetch_sts_token(access_key_id, access_key_secret, role_arn):
        clt = client.AcsClient(access_key_id, access_key_secret, OSS_REGION)
        req = AssumeRoleRequest.AssumeRoleRequest()

        req.set_accept_format('json')
        req.set_RoleArn(role_arn)
        req.set_RoleSessionName('oss-python-sdk-test')

        body = clt.do_action_with_exception(req)

        j = json.loads(oss2.to_unicode(body))

        token = StsToken()

        token.access_key_id = j['Credentials']['AccessKeyId']
        token.access_key_secret = j['Credentials']['AccessKeySecret']
        token.security_token = j['Credentials']['SecurityToken']
        token.request_id = j['RequestId']
        token.expiration = oss2.utils.to_unixtime(j['Credentials']['Expiration'], '%Y-%m-%dT%H:%M:%SZ')

        return token


    class TestSTSAuth(oss2.StsAuth):
        def __init__(self, access_key_id, access_key_secret, security_token):
            super(TestSTSAuth, self).__init__(access_key_id,
                                              access_key_secret,
                                              security_token,
                                              os.getenv('OSS_TEST_AUTH_VERSION'))

    class TestSts(unittest.TestCase):
        def setUp(self):
            self.bucket = None
            self.key_list = []
            self.prefix = 'sts-' + random_string(8) + '/'

        def tearDown(self):
            if self.bucket is not None:
                clean_and_delete_bucket(self.bucket)

        def random_key(self, suffix=''):
            key = self.prefix + random_string(12) + suffix
            self.key_list.append(key)

            return key

        def init_bucket(self):
            self.token = fetch_sts_token(OSS_STS_ID, OSS_STS_KEY, OSS_STS_ARN)

            auth = oss2.StsAuth(self.token.access_key_id, self.token.access_key_secret, self.token.security_token)
            self.bucket = oss2.Bucket(auth, OSS_ENDPOINT, OSS_BUCKET)
            self.bucket.create_bucket()

        def test_object(self):
            self.init_bucket()

            key = self.random_key()
            content = b'hello world'

            self.bucket.put_object(key, content)
            self.assertEqual(self.bucket.get_object(key).read(), content)

            self.bucket.delete_object(key)

        def test_bucket(self):
            self.init_bucket()

            # just make sure no exception being thrown
            self.bucket.get_bucket_referer()

        def test_url(self):
            self.init_bucket()

            key = self.random_key()
            content = b'Ali Baba'

            self.bucket.put_object(key, content)
            url = self.bucket.sign_url('GET', key, 60, params={'para1':'test'})

            resp = requests.get(url)
            self.assertEqual(content, resp.content)

        def test_rtmp(self):
            channel_name = 'test-sign-rtmp-url'
            
            self.init_bucket()
            
            self.bucket.list_live_channel()
            
            url = self.bucket.sign_rtmp_url(channel_name, 'test.m3u8', 3600)
            self.assertTrue('security-token=' in url)

    class TestSign(TestSts):
        """
            这个类主要是用来增加测试覆盖率，当环境变量为oss2.AUTH_VERSION_2，则重新设置为oss2.AUTH_VERSION_1再运行TestSts，反之亦然
        """
        def __init__(self, *args, **kwargs):
            super(TestSign, self).__init__(*args, **kwargs)

        def setUp(self):
            if os.getenv('OSS_TEST_AUTH_VERSION') == oss2.AUTH_VERSION_2:
                os.environ['OSS_TEST_AUTH_VERSION'] = oss2.AUTH_VERSION_1
            else:
                os.environ['OSS_TEST_AUTH_VERSION'] = oss2.AUTH_VERSION_2
            super(TestSign, self).setUp()

        def tearDown(self):
            if os.getenv('OSS_TEST_AUTH_VERSION') == oss2.AUTH_VERSION_2:
                os.environ['OSS_TEST_AUTH_VERSION'] = oss2.AUTH_VERSION_1
            else:
                os.environ['OSS_TEST_AUTH_VERSION'] = oss2.AUTH_VERSION_2
            super(TestSign, self).tearDown()


# -*- coding: utf-8 -*-

import time
import requests
import json
import logging
import threading
from .exceptions import ClientError
from .utils import to_unixtime
from .compat import to_unicode

logger = logging.getLogger(__name__)


class Credentials(object):
    def __init__(self, access_key_id="", access_key_secret="", security_token=""):
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.security_token = security_token

    def get_access_key_id(self):
        return self.access_key_id

    def get_access_key_secret(self):
        return self.access_key_secret

    def get_security_token(self):
        return self.security_token


DEFAULT_ECS_SESSION_TOKEN_DURATION_SECONDS = 3600 * 6
DEFAULT_ECS_SESSION_EXPIRED_FACTOR = 0.85


class EcsRamRoleCredential(Credentials):
    def __init__(self,
                 access_key_id,
                 access_key_secret,
                 security_token,
                 expiration,
                 duration,
                 expired_factor=None):
        self.access_key_id = access_key_id
        self.access_key_secret = access_key_secret
        self.security_token = security_token
        self.expiration = expiration
        self.duration = duration
        self.expired_factor = expired_factor or DEFAULT_ECS_SESSION_EXPIRED_FACTOR

    def get_access_key_id(self):
        return self.access_key_id

    def get_access_key_secret(self):
        return self.access_key_secret

    def get_security_token(self):
        return self.security_token

    def will_soon_expire(self):
        now = int(time.time())
        return self.duration * (1.0 - self.expired_factor) > self.expiration - now


class CredentialsProvider(object):
    def get_credentials(self):
        return


class StaticCredentialsProvider(CredentialsProvider):
    def __init__(self, access_key_id="", access_key_secret="", security_token=""):
        self.credentials = Credentials(access_key_id, access_key_secret, security_token)

    def get_credentials(self):
        return self.credentials


class EcsRamRoleCredentialsProvider(CredentialsProvider):
    def __init__(self, auth_host, max_retries=3, timeout=10):
        self.fetcher = EcsRamRoleCredentialsFetcher(auth_host)
        self.max_retries = max_retries
        self.timeout = timeout
        self.credentials = None
        self.__lock = threading.Lock()

    def get_credentials(self):
        if self.credentials is None or self.credentials.will_soon_expire():
            with self.__lock:
                if self.credentials is None or self.credentials.will_soon_expire():
                    try:
                        self.credentials = self.fetcher.fetch(self.max_retries, self.timeout)
                    except Exception as e:
                        logger.error("Exception: {0}".format(e))
                        if self.credentials is None:
                            raise

        return self.credentials


class EcsRamRoleCredentialsFetcher(object):
    def __init__(self, auth_host):
        self.auth_host = auth_host

    def fetch(self, retry_times=3, timeout=10):
        for i in range(0, retry_times):
            try:
                response = requests.get(self.auth_host, timeout=timeout)
                if response.status_code != 200:
                    raise ClientError(
                        "Failed to fetch credentials url, http code:{0}, msg:{1}".format(response.status_code,
                                                                                         response.text))
                dic = json.loads(to_unicode(response.content))
                code = dic.get('Code')
                access_key_id = dic.get('AccessKeyId')
                access_key_secret = dic.get('AccessKeySecret')
                security_token = dic.get('SecurityToken')
                expiration_date = dic.get('Expiration')
                last_updated_date = dic.get('LastUpdated')

                if code != "Success":
                    raise ClientError("Get credentials from ECS metadata service error, code: {0}".format(code))

                expiration_stamp = to_unixtime(expiration_date, "%Y-%m-%dT%H:%M:%SZ")
                duration = DEFAULT_ECS_SESSION_TOKEN_DURATION_SECONDS
                if last_updated_date is not None:
                    last_updated_stamp = to_unixtime(last_updated_date, "%Y-%m-%dT%H:%M:%SZ")
                    duration = expiration_stamp - last_updated_stamp
                return EcsRamRoleCredential(access_key_id, access_key_secret, security_token, expiration_stamp,
                                            duration, DEFAULT_ECS_SESSION_EXPIRED_FACTOR)
            except Exception as e:
                if i == retry_times - 1:
                    logger.error("Exception: {0}".format(e))
                    raise ClientError("Failed to get credentials from ECS metadata service. {0}".format(e))



# -*- coding: utf-8 -*-

from .common import *
from oss2.models import UserQosInfo, BucketQosInfo

class TestQosInfo(OssTestCase):
    def test_get_user_qos_info(self):
        service = oss2.Service(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT)

        result = service.get_user_qos_info()
        self.assertEqual(result.status, 200)
        self.assertTrue(result.region is not None)
        self.assertTrue(result.total_upload_bw is not None)
        self.assertTrue(result.intranet_upload_bw is not None)
        self.assertTrue(result.extranet_upload_bw is not None)
        self.assertTrue(result.total_download_bw is not None)
        self.assertTrue(result.intranet_download_bw is not None)
        self.assertTrue(result.extranet_download_bw is not None)
        self.assertTrue(result.total_qps is not None)
        self.assertTrue(result.intranet_qps is not None)
        self.assertTrue(result.extranet_qps is not None)

    def test_put_bucket_qos_info_with_all_args(self):
        bucket_qos_info = BucketQosInfo(
                    total_upload_bw = -1,
                    intranet_upload_bw = 2,
                    extranet_upload_bw = 2,
                    total_download_bw = -1,
                    intranet_download_bw = -1,
                    extranet_download_bw = -1,
                    total_qps = -1,
                    intranet_qps = -1,
                    extranet_qps = -1)

        result = self.bucket.put_bucket_qos_info(bucket_qos_info)
        self.assertEqual(result.status, 200)

        result = self.bucket.get_bucket_qos_info()
        self.assertEqual(result.status, 200)
        self.assertEqual(result.total_upload_bw, bucket_qos_info.total_upload_bw)
        self.assertEqual(result.intranet_upload_bw, bucket_qos_info.intranet_upload_bw)
        self.assertEqual(result.extranet_upload_bw, bucket_qos_info.extranet_upload_bw)
        self.assertEqual(result.total_download_bw, bucket_qos_info.total_download_bw)
        self.assertEqual(result.intranet_download_bw, bucket_qos_info.intranet_download_bw)
        self.assertEqual(result.extranet_download_bw, bucket_qos_info.extranet_download_bw)
        self.assertEqual(result.total_qps, bucket_qos_info.total_qps)
        self.assertEqual(result.intranet_qps, bucket_qos_info.intranet_qps)
        self.assertEqual(result.extranet_qps, bucket_qos_info.extranet_qps)

        result = self.bucket.delete_bucket_qos_info()
        self.assertEqual(result.status, 204)

    def test_put_bucket_qos_info_with_none_args(self):
        bucket_qos_info = BucketQosInfo()

        # put bucket qos info without args setting, the default value of -1 will be returned.
        result = self.bucket.put_bucket_qos_info(bucket_qos_info)
        self.assertEqual(result.status, 200)

        result = self.bucket.get_bucket_qos_info()
        self.assertEqual(result.total_upload_bw, -1)
        self.assertEqual(result.intranet_upload_bw, -1)
        self.assertEqual(result.extranet_upload_bw, -1)
        self.assertEqual(result.total_download_bw, -1)
        self.assertEqual(result.intranet_download_bw, -1)
        self.assertEqual(result.extranet_download_bw, -1)
        self.assertEqual(result.total_qps, -1)
        self.assertEqual(result.intranet_qps, -1)
        self.assertEqual(result.extranet_qps, -1)

        result = self.bucket.delete_bucket_qos_info()
        self.assertEqual(result.status, 204)

    def test_put_bucket_qos_info_illegal_args(self):
        service = oss2.Service(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT)
        user_qos_info = service.get_user_qos_info()
        self.assertTrue(user_qos_info.total_upload_bw > 0, 'user_qos_info.total_upload_bw should be > 0')
        self.assertTrue(user_qos_info.total_download_bw > 0, 'user_qos_info.total_download_bw should be > 0')

        # total_upload_bw > user_qos_info.total_upload_bw, should be failed.
        total_upload_bw = user_qos_info.total_upload_bw + 1
        bucket_qos_info = BucketQosInfo(total_upload_bw=total_upload_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        # intranet_upload_bw > total_upload_bw, should be failed.
        total_upload_bw = user_qos_info.total_upload_bw
        intranet_upload_bw = total_upload_bw + 1
        bucket_qos_info = BucketQosInfo(total_upload_bw=total_upload_bw, intranet_upload_bw=intranet_upload_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        # extranet_upload_bw > total_upload_bw, should be failed.
        total_upload_bw = user_qos_info.total_upload_bw
        extranet_upload_bw = total_upload_bw + 1
        bucket_qos_info = BucketQosInfo(total_upload_bw, extranet_upload_bw=extranet_upload_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        # total_download_bw > user_qos_info.total_upload_bw, should be failed.
        total_download_bw = user_qos_info.total_download_bw + 1
        bucket_qos_info = BucketQosInfo(total_download_bw=total_download_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        # intranet_download_bw > total_download_bw, should be failed.
        total_download_bw = user_qos_info.total_download_bw
        intranet_download_bw = total_download_bw + 1
        bucket_qos_info = BucketQosInfo(total_download_bw=total_download_bw, intranet_download_bw=intranet_download_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        # extranet_download_bw > total_download_bw, should be failed.
        total_download_bw = user_qos_info.total_download_bw
        extranet_download_bw = total_download_bw + 1
        bucket_qos_info = BucketQosInfo(total_download_bw=total_download_bw, extranet_download_bw=extranet_download_bw)
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

        #self.assertTrue(user_qos_info.total_qps > 0, 'user_qos_info.total_qps should be > 0')
        if user_qos_info.total_qps > 0 :
            # total_qps > user_qos_info.total_qps, should be failed.
            total_qps = user_qos_info.total_qps + 1
            bucket_qos_info = BucketQosInfo(total_qps=total_qps)
            self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

            # intranet_qps > total_qps, should be failed.
            total_qps = total_qps
            intranet_qps = total_qps + 1
            bucket_qos_info = BucketQosInfo(total_qps=total_qps, intranet_qps=intranet_qps)
            self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)

            # extranet_qps > total_qps, should be failed.
            total_qps = total_qps
            extranet_qps = total_qps + 1
            bucket_qos_info = BucketQosInfo(extranet_qps=extranet_qps)
            self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_qos_info, bucket_qos_info)
        else:
            self.assertTrue(user_qos_info.total_qps == -1, 'default user_qos_info.total_qps is -1')

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import os
import oss2
import requests
import datetime
import time
import hashlib
import hmac


# 下面的代码展示了使用OSS V2签名算法来对请求进行签名


# 首先，初始化AccessKeyId、AccessKeySecret和Endpoint.
# 你可以通过设置环境变量来设置access_key_id等, 或者直接使用真实access_key_id替换'<Your AccessKeyId>'等
#
# 以杭州(华东1)作为例子, endpoint应该是
#   http://oss-cn-hangzhou.aliyuncs.com
#   https://oss-cn-hangzhou.aliyuncs.com
# 对HTTP和HTTPS请求，同样的处理
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<Your AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<Your AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<Your Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<Your Endpoint>')


if not endpoint.startswith('http://') and not endpoint.startswith('https://'):
    endpoint = 'http://' + endpoint


# 验证access_key_id和其他参数都被合理地初始化
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, 'Please set variable: ' + param


# 创建一个AuthV2对象，这样我们就可以用V2算法来签名请求。也可以使用oss2.make_auth函数，默认采用V1算法
auth = oss2.AuthV2(access_key_id, access_key_secret)
# auth = oss2.make_auth(access_key_id, access_key_secret, oss2.AUTH_VERSION_2)

# 创建一个Bucket，利用它进行所有bucket与object相关操作
bucket = oss2.Bucket(auth, endpoint, bucket_name)

content = b'Never give up. - Jack Ma'

# 上传一个Object
bucket.put_object('motto.txt', content)

# 下载一个object
result = bucket.get_object('motto.txt')

assert result.read() == content

# 生成一个签名的URL，将在60秒后过期
url = bucket.sign_url('GET', 'motto.txt', 60)

print(url)


# 人工构造一个使用V2签名的请求
key = 'object-from-post.txt'

boundary = 'arbitraryboundaryvalue'
headers = {'Content-Type': 'multipart/form-data; boundary=' + boundary}
encoded_policy = oss2.utils.b64encode_as_string(oss2.to_bytes('{ "expiration": "%s","conditions": [["starts-with", "$key", ""]]}'
         % oss2.date_to_iso8601(datetime.datetime.utcfromtimestamp(int(time.time()) + 60))))

digest = hmac.new(oss2.to_bytes(access_key_secret), oss2.to_bytes(encoded_policy), hashlib.sha256).digest()
signature = oss2.utils.b64encode_as_string(digest)

form_fields = {
    'x-oss-signature-version': 'OSS2',
    'x-oss-signature': signature,
    'x-oss-access-key-id': access_key_id,
    'policy': encoded_policy,
    'key': key,
}

# 对象的内容
content = 'file content for post object request'

body = ''

for k, v in form_fields.items():
    body += '--%s\r\nContent-Disposition: form-data; name="%s"\r\n\r\n%s\r\n' % (boundary, k, v)

body += '--%s\r\nContent-Disposition: form-data; name="file"; filename="%s"\r\n\r\n%s\r\n' % (boundary, key, content)
body += '--%s\r\nContent-Disposition: form-data; name="submit"\r\n\r\nUpload to OSS\r\n--%s--\r\n' % (boundary, boundary)

p = oss2.urlparse(endpoint)
requests.post('%s://%s.%s' % (p.scheme, bucket_name, p.netloc), data=body, headers=headers)
